from qtpy import QtGui, QtCore, QtWidgets
import sharppy.viz as viz
import sharppy.viz.preferences as preferences
from sharppy.io.spc_decoder import SPCDecoder
import pytest
import os
import sys
import numpy as np

""" plotText() and plotSkewT keep failing """
## Travis CI allows for a psuedo X-window editor to run if you are running
## a Linux image.  So that means that only on Linux can we run GUI tests.
## So, I've created a decorator to only run these tests if the DISPLAY_AVAIL
## variable is set.

dec = SPCDecoder('examples/data/14061619.OAX')
prof_coll = dec.getProfiles()
prof = prof_coll.getCurrentProfs()['']

if QtWidgets.QApplication.instance() is None:
    app = QtWidgets.QApplication([])
else:
    app = QtWidgets.QApplication.instance()

@pytest.mark.skipif(True, reason="DISPLAY not set")
# @pytest.mark.skipif("DISPLAY_AVAIL" in os.environ and os.environ["DISPLAY_AVAIL"] == 'NO', reason="DISPLAY not set")
def test_insets():
    insets = [viz.fire.plotFire,
              viz.winter.plotWinter,
              viz.kinematics.plotKinematics,
              viz.stp.plotSTP,
              viz.ship.plotSHIP,
              viz.vrot.plotVROT,
              viz.analogues.plotAnalogues,
              viz.stpef.plotSTPEF]
    names = ['fire', 'winter', 'kinematics', 'stp', 'ship', 'vrot', 'sars', 'stpef']
    for inset, name in zip(insets, names):
        print("Testing:", str(inset))
        if inset is viz.thermo.plotText:
            test = inset(['SFC', 'ML', 'MU', 'FCST'])
        else:
            test = inset()
        test.setProf(prof)
        test.setGeometry(50,50,293,195)
        test.plotBitMap.save(name + '_test.png', format='png')
        del test

    ens = viz.ensemble.plotENS()
    ens.addProfileCollection(prof_coll)
    ens.setActiveCollection(0)

# @pytest.mark.skipif("DISPLAY_AVAIL" in os.environ and os.environ["DISPLAY_AVAIL"] == 'NO', reason="DISPLAY not set")
@pytest.mark.skipif(True, reason="DISPLAY not set")
def test_hodo():
    hodo = viz.hodo.plotHodo

    s = hodo()
    s.addProfileCollection(prof_coll)
    s.setActiveCollection(0)
    #s.setMWCenter()
    #s.setSRCenter()
    #s.setDeviant('left')
    s.plotBitMap.save('hodo.png', format='png')

# @pytest.mark.skipif("DISPLAY_AVAIL" in os.environ and os.environ["DISPLAY_AVAIL"] == 'NO', reason="DISPLAY not set")
@pytest.mark.skipif(True, reason="DISPLAY not set")
def test_skew():
    skew = viz.skew.plotSkewT
    #s = skew()
    #s.addProfileCollection(prof_coll)
    #s.setActiveCollection(0)
    #s.plotBitMap.save('skew.png', format='png')

# @pytest.mark.skipif("DISPLAY_AVAIL" in os.environ and os.environ["DISPLAY_AVAIL"] == 'NO', reason="DISPLAY not set")
@pytest.mark.skipif(True, reason="DISPLAY not set")
def test_smaller_insets():
    insets = [viz.speed.plotSpeed,
              viz.advection.plotAdvection,
              viz.watch.plotWatch,
              viz.srwinds.plotWinds,
              viz.slinky.plotSlinky,
              viz.thetae.plotThetae]
    names = ['speed', 'advection', 'watch', 'srwinds', 'slinky', 'thetae']
    for inset, name in zip(insets, names):
        print("Testing:", str(inset))
        test = inset()
        test.setProf(prof)
        if name == 'slinky':
            test.setParcel(prof.mupcl)
            test.setDeviant('left')
        test.setGeometry(50,50,293,195)
        test.plotBitMap.save(name + '_test.png', format='png')
        del test

    test = viz.generic.plotGeneric(np.asarray([1,2]),np.asarray([1,2]))
    del test

# @pytest.mark.skipif("DISPLAY_AVAIL" in os.environ and os.environ["DISPLAY_AVAIL"] == 'NO', reason="DISPLAY not set")
@pytest.mark.skipif(True, reason="DISPLAY not set")
def test_mapper():
    mapper = viz.map.Mapper(-97,35)
    assert mapper.getLambda0() == -97
    assert mapper.getPhi0() == 35
    mapper.setLambda0(-97)
    for proj in ['npstere', 'spstere', 'merc']:
        mapper.setProjection(proj)
        assert mapper.getProjection() == proj
        mapper.getCoordPaths()

app.quit()
del app



''' Frequently used meteorological constants '''

__all__ = ['MISSING', 'ROCP', 'ZEROCNK', 'G', 'TOL', 'WHITE', 'RED',
           'ORANGE', 'YELLOW', 'MAGENTA', 'DBROWN', 'LBROWN', 'LBLUE',
           'CYAN', 'BLACK', 'GREEN', 'DGREEN', 'HAINES_HIGH', 'HAINES_MID',
           'HAINES_LOW']

# Meteorological Constants
MISSING = -9999.0       # Missing Flag
ROCP = 0.28571426       # R over Cp
ZEROCNK = 273.15        # Zero Celsius in Kelvins
G = 9.80665             # Gravity
TOL = 1e-10             # Floating Point Tolerance

# Color code constants for easy access
WHITE = '#FFFFFF'
BLACK = '#000000'
RED = '#FF0000'
ORANGE = '#FF4000'
YELLOW = '#FFFF00'
MAGENTA = '#E700DF'
DBROWN = '#775000'
LBROWN = '#996600'
LBLUE = '#06B5FF'
CYAN = '#00FFFF'
GREEN = '#00FF00'
DGREEN = '#006000'

# Haines Index elevation constants for easy access
HAINES_HIGH = 2
HAINES_MID  = 1
HAINES_LOW  = 0





import numpy as np
import os
from qtpy import QtGui, QtCore, QtWidgets
import sharppy.sharptab as tab
import sharppy.databases.inset_data as inset_data
from sharppy.sharptab.constants import *
import platform

## routine written by Kelton Halbert and Greg Blumberg
## keltonhalbert@ou.edu and wblumberg@ou.edu

__all__ = ['backgroundSTP', 'plotSTP']

class backgroundSTP(QtWidgets.QFrame):
    '''
    Draw the background frame and lines for the STP plot frame
    '''
    def __init__(self):
        super(backgroundSTP, self).__init__()
        self.initUI()


    def initUI(self):
        ## window configuration settings,
        ## sich as padding, width, height, and
        ## min/max plot axes
        self.setStyleSheet("QFrame {"
            "  background-color: rgb(0, 0, 0);"
            "  border-width: 1px;"
            "  border-style: solid;"
            "  border-color: #3399CC;}")
        self.textpad = 5
        self.font_ratio = 0.0512
        fsize1 = round(self.size().height() * self.font_ratio) + 2
        fsize2 = round(self.size().height() * self.font_ratio)
        self.plot_font = QtGui.QFont('Helvetica', fsize1 )
        self.box_font = QtGui.QFont('Helvetica', fsize2)
        self.plot_metrics = QtGui.QFontMetrics( self.plot_font )
        self.box_metrics = QtGui.QFontMetrics(self.box_font)
        if platform.system() == "Windows":
            fsize1 -= self.plot_metrics.descent()
            fsize2 -= self.box_metrics.descent()

            self.plot_font = QtGui.QFont('Helvetica', fsize1 )
            self.box_font = QtGui.QFont('Helvetica', fsize2)
            self.plot_metrics = QtGui.QFontMetrics( self.plot_font )
            self.box_metrics = QtGui.QFontMetrics(self.box_font)
        self.plot_height = self.plot_metrics.xHeight()# + self.textpad
        self.box_height = self.box_metrics.xHeight() + self.textpad
        self.tpad = self.plot_height + 15; 
        self.bpad = self.plot_height + 2
        self.lpad = 0.; self.rpad = 0.
        self.wid = self.size().width() - self.rpad
        self.hgt = self.size().height() - self.bpad
        self.tlx = self.rpad; self.tly = self.tpad;
        self.brx = self.wid; 
        self.bry = self.hgt - self.bpad#+ round(self.font_ratio * self.hgt)
        self.stpmax = 11.; self.stpmin = 0.
        self.plotBitMap = QtGui.QPixmap(self.width()-2, self.height()-2)
        self.plotBitMap.fill(self.bg_color)
        self.plotBackground()

    def resizeEvent(self, e):
        '''
        Handles the event the window is resized
        '''
        self.initUI()
    
    def plotBackground(self):
        '''
        Handles painting the frame.
        '''
        ## initialize a painter object and draw the frame
        qp = QtGui.QPainter()
        qp.begin(self.plotBitMap)
        qp.setRenderHint(qp.Antialiasing)
        qp.setRenderHint(qp.TextAntialiasing)
        self.draw_frame(qp)
        qp.end()

    def draw_frame(self, qp):
        '''
        Draw the background frame.
        qp: QtGui.QPainter object
        '''
        ## set a new pen to draw with
        pen = QtGui.QPen(self.fg_color, 2, QtCore.Qt.SolidLine)
        qp.setPen(pen)
        qp.setFont(self.plot_font)
        # Left, top, width, height
        rect1 = QtCore.QRectF(0,5, self.brx, self.plot_height)
        qp.drawText(rect1, QtCore.Qt.TextDontClip | QtCore.Qt.AlignCenter,
            'Effective Layer STP (with CIN)')

        pen = QtGui.QPen(QtCore.Qt.blue, 1, QtCore.Qt.DashLine)
        qp.setPen(pen)

        ytick_fontsize = round(self.font_ratio * self.hgt) + 1
        y_ticks_font = QtGui.QFont('Helvetica', ytick_fontsize)
        qp.setFont(y_ticks_font)
        stp_inset_data = inset_data.stpData()
        texts = stp_inset_data['stp_ytexts']
        # Plot the y-tick labels for the box and whisker plots
        for i in range(len(texts)):
            pen = QtGui.QPen(self.line_color, 1, QtCore.Qt.DashLine)
            qp.setPen(pen)
            tick_pxl = self.stp_to_pix(int(texts[i]))
            qp.drawLine(self.tlx, tick_pxl, self.brx, tick_pxl)
            color = QtGui.QColor(self.bg_color)
            pen = QtGui.QPen(color, 1, QtCore.Qt.SolidLine)
            qp.setPen(pen)
            ypos = tick_pxl - ytick_fontsize/2
            rect = QtCore.QRect(self.tlx, ypos, 20, ytick_fontsize)
            pen = QtGui.QPen(self.fg_color, 1, QtCore.Qt.SolidLine)
            qp.setPen(pen)
            qp.drawText(rect, QtCore.Qt.TextDontClip | QtCore.Qt.AlignCenter, texts[i])
        
        # Draw the box and whisker plots and the x-tick labels
        ef = stp_inset_data['ef']
        width = self.brx / 14
        spacing = self.brx / 7
        center = np.arange(spacing, self.brx, spacing)
        texts = stp_inset_data['stp_xtexts']
        ef = self.stp_to_pix(ef)
        qp.setFont(QtGui.QFont('Helvetica', round(self.font_ratio * self.hgt)))
        for i in range(ef.shape[0]):
            # Set green pen to draw box and whisker plots 
            pen = QtGui.QPen(self.box_color, 2, QtCore.Qt.SolidLine)
            qp.setPen(pen)
            # Draw lower whisker
            qp.drawLine(center[i], ef[i,0], center[i], ef[i,1])
            # Draw box
            qp.drawLine(center[i] - width/2., ef[i,3], center[i] + width/2., ef[i,3])
            qp.drawLine(center[i] - width/2., ef[i,1], center[i] + width/2., ef[i,1])
            qp.drawLine(center[i] - width/2., ef[i,1], center[i] - width/2., ef[i,3])
            qp.drawLine(center[i] + width/2., ef[i,1], center[i] + width/2., ef[i,3])
            # Draw median
            qp.drawLine(center[i] - width/2., ef[i,2], center[i] + width/2., ef[i,2])
            # Draw upper whisker
            qp.drawLine(center[i], ef[i,3], center[i], ef[i,4])
            # Set black transparent pen to draw a rectangle
            color = QtGui.QColor(self.bg_color)
            color.setAlpha(0)
            pen = QtGui.QPen(color, 1, QtCore.Qt.SolidLine)
            rect = QtCore.QRectF(center[i] - width/2., self.bry + round(self.bpad/2), width, self.bpad)

            # Change to a white pen to draw the text below the box and whisker plot
            pen = QtGui.QPen(self.fg_color, 1, QtCore.Qt.SolidLine)
            qp.setPen(pen)
            qp.drawText(rect, QtCore.Qt.TextDontClip | QtCore.Qt.AlignCenter, texts[i])
            #qp.drawText(rect, QtCore.Qt.AlignCenter, texts[i])

    def stp_to_pix(self, stp):
        scl1 = self.stpmax - self.stpmin
        scl2 = self.stpmin + stp
        return self.bry - (scl2 / scl1) * (self.bry - self.tpad)


class plotSTP(backgroundSTP):
    '''
    Plot the data on the frame. Inherits the background class that
    plots the frame.
    '''
    def __init__(self):
        self.bg_color = QtGui.QColor('#000000')
        self.fg_color = QtGui.QColor('#ffffff')
        self.box_color = QtGui.QColor('#00ff00')
        self.line_color = QtGui.QColor('#0080ff')

        self.alert_colors = [
            QtGui.QColor('#775000'),
            QtGui.QColor('#996600'),
            QtGui.QColor('#ffffff'),
            QtGui.QColor('#ffff00'),
            QtGui.QColor('#ff0000'),
            QtGui.QColor('#e700df'),
        ]

        self.use_left = False

        super(plotSTP, self).__init__()
        self.prof = None

    def setProf(self, prof):
        self.prof = prof

        self.mlcape = prof.mlpcl.bplus
        self.mllcl = prof.mlpcl.lclhght
        self.ebwd = prof.ebwspd

        if self.use_left:
            self.esrh = prof.left_esrh[0]
            self.stpc = prof.left_stp_cin
            self.stpf = prof.left_stp_fixed
        else:
            self.esrh = prof.right_esrh[0]
            self.stpc = prof.right_stp_cin
            self.stpf = prof.right_stp_fixed

        if self.prof.latitude < 0:
            self.esrh = -self.esrh
            self.stpc = -self.stpc
            self.stpf = -self.stpf

        ## get the probabilities
        self.cape_p, self.cape_c = self.cape_prob(self.mlcape)
        self.lcl_p, self.lcl_c = self.lcl_prob(self.mllcl)
        self.esrh_p, self.esrh_c = self.esrh_prob(self.esrh)
        self.ebwd_p, self.ebwd_c = self.ebwd_prob(self.ebwd)
        self.stpc_p, self.stpc_c = self.stpc_prob(self.stpc)
        self.stpf_p, self.stpf_c = self.stpf_prob(self.stpf)

        #self.ylast = self.tpad
        self.clearData()
        self.plotBackground()
        self.plotData()
        self.update()

    def setPreferences(self, update_gui=True, **prefs):
        self.bg_color = QtGui.QColor(prefs['bg_color'])
        self.fg_color = QtGui.QColor(prefs['fg_color'])
        self.box_color = QtGui.QColor(prefs['stp_box_color'])
        self.line_color = QtGui.QColor(prefs['stp_line_color'])

        self.alert_colors = [
            QtGui.QColor(prefs['alert_l1_color']),
            QtGui.QColor(prefs['alert_l2_color']),
            QtGui.QColor(prefs['alert_l3_color']),
            QtGui.QColor(prefs['alert_l4_color']),
            QtGui.QColor(prefs['alert_l5_color']),
            QtGui.QColor(prefs['alert_l6_color']),
        ]

        if update_gui:
            if self.use_left:
                self.esrh = self.prof.left_esrh[0]
                self.stpc = self.prof.left_stp_cin
                self.stpf = self.prof.left_stp_fixed
            else:
                self.esrh = self.prof.right_esrh[0]
                self.stpc = self.prof.right_stp_cin
                self.stpf = self.prof.right_stp_fixed

            if self.prof is not None and  self.prof.latitude < 0:
                self.esrh = -self.esrh
                self.stpc = -self.stpc
                self.stpf = -self.stpf

            self.cape_p, self.cape_c = self.cape_prob(self.mlcape)
            self.lcl_p, self.lcl_c = self.lcl_prob(self.mllcl)
            self.esrh_p, self.esrh_c = self.esrh_prob(self.esrh)
            self.ebwd_p, self.ebwd_c = self.ebwd_prob(self.ebwd)
            self.stpc_p, self.stpc_c = self.stpc_prob(self.stpc)
            self.stpf_p, self.stpf_c = self.stpf_prob(self.stpf)

            self.clearData()
            self.plotBackground()
            self.plotData()
            self.update()

    def setDeviant(self, deviant):
        self.use_left = deviant == 'left'

        if self.use_left:
            self.esrh = self.prof.left_esrh[0]
            self.stpc = self.prof.left_stp_cin
            self.stpf = self.prof.left_stp_fixed
        else:
            self.esrh = self.prof.right_esrh[0]
            self.stpc = self.prof.right_stp_cin
            self.stpf = self.prof.right_stp_fixed

        if self.prof.latitude < 0:
            self.esrh = -self.esrh
            self.stpc = -self.stpc
            self.stpf = -self.stpf

        self.esrh_p, self.esrh_c = self.esrh_prob(self.esrh)
        self.stpc_p, self.stpc_c = self.stpc_prob(self.stpc)
        self.stpf_p, self.stpf_c = self.stpf_prob(self.stpf)

        self.clearData()
        self.plotBackground()
        self.plotData()
        self.update()

    def cape_prob(self, cape):
        if cape == 0.:
            prob = 0.00
            color = self.alert_colors[0]
        elif cape  > 0. and cape < 250.:
            prob = .12
            color = self.alert_colors[1]
        elif cape >= 250. and cape < 500.:
            prob = .14
            color = self.alert_colors[2]
        elif cape >= 500. and cape < 1000.:
            prob = .16
            color = self.alert_colors[2]
        elif cape >= 1000. and cape < 1500.:
            prob = .15
            color = self.alert_colors[2]
        elif cape >= 1500. and cape < 2000.:
            prob = .13
            color = self.alert_colors[2]
        elif cape >= 2000. and cape < 2500.:
            prob = .14
            color = self.alert_colors[2]
        elif cape >= 2500. and cape < 3000.:
            prob = .18
            color = self.alert_colors[3]
        elif cape >= 3000. and cape < 4000.:
            prob = .20
            color = self.alert_colors[3]
        elif cape >= 4000.:
            prob = .16
            color = self.alert_colors[3]
        else:
            prob = np.ma.masked
            color = self.alert_colors[0]
        return prob, color
    
    def lcl_prob(self, lcl):
        if lcl < 750.:
            prob = .19
            color = self.alert_colors[3]
        elif lcl >= 750. and lcl < 1000.:
            prob = .19
            color = self.alert_colors[3]
        elif lcl >= 1000. and lcl < 1250.:
            prob = .15
            color = self.alert_colors[2] 
        elif lcl >= 1250. and lcl < 1500.:
            prob = .10
            color = self.alert_colors[1]
        elif lcl >= 1500. and lcl < 1750:
            prob = .06
            color = self.alert_colors[0]
        elif lcl >= 1750. and lcl < 2000.:
            prob = .06
            color = self.alert_colors[0]
        elif lcl >= 2000. and lcl < 2500.:
            prob = .02
            color = self.alert_colors[0]
        elif lcl >= 2500:
            prob = 0.0
            color = self.alert_colors[0]
        else:
            prob = np.ma.masked
            color = self.alert_colors[0]
        return prob, color
    
    def esrh_prob(self, esrh):
        if esrh < 50.:
            prob = .06
            color = self.alert_colors[0]
        elif esrh >= 50. and esrh < 100.:
            prob = .06
            color = self.alert_colors[0]
        elif esrh >= 100. and esrh < 200.:
            prob = .08
            color = self.alert_colors[1]
        elif esrh >= 200. and esrh < 300:
            prob = .14
            color = self.alert_colors[2]
        elif esrh >= 300. and esrh < 400.:
            prob = .20
            color = self.alert_colors[3]
        elif esrh >= 400. and esrh < 500.:
            prob = .27
            color = self.alert_colors[3]
        elif esrh >= 500. and esrh < 600:
            prob = .38
            color = self.alert_colors[4]
        elif esrh >= 600. and esrh < 700.:
            prob = .37
            color = self.alert_colors[4]
        elif esrh >= 700:
            prob = .42
            color = self.alert_colors[4]
        else:
            prob = np.ma.masked
            color = self.alert_colors[0]
        return prob, color
    
    def ebwd_prob(self, ebwd):
        if ebwd == 0.:
            prob = 0.0
            color = self.alert_colors[0]
        elif ebwd >= .01 and ebwd < 20.:
            prob = .03
            color = self.alert_colors[0]
        elif ebwd >= 20. and ebwd < 30.:
            prob = .05
            color = self.alert_colors[0]
        elif ebwd >= 30. and ebwd < 40.:
            prob = .06
            color = self.alert_colors[0]
        elif ebwd >= 40. and ebwd < 50.:
            prob = .12
            color = self.alert_colors[1]
        elif ebwd >= 50. and ebwd < 60.:
            prob = .19
            color = self.alert_colors[3]
        elif ebwd >= 60. and ebwd < 70.:
            prob = .27
            color = self.alert_colors[3]
        elif ebwd >= 70. and ebwd < 80.:
            prob = .36
            color = self.alert_colors[4]
        elif ebwd >= 80.:
            prob = .26
            color = self.alert_colors[3]
        else:
            prob = np.ma.masked
            color = self.alert_colors[0]
        return prob, color

    def stpc_prob(self, stpc):
        if stpc < .1:
            prob = .06
            color = self.alert_colors[0]
        elif stpc >= .1 and stpc < .50:
            prob = .08
            color = self.alert_colors[1]
        elif stpc >= .5 and stpc < 1.0:
            prob = .12
            color = self.alert_colors[1]
        elif stpc >= 1. and stpc < 2.:
            prob = .17
            color = self.alert_colors[2]
        elif stpc >= 2. and stpc < 4.:
            prob = .25
            color = self.alert_colors[3]
        elif stpc >= 4. and stpc < 6.:
            prob = .32
            color = self.alert_colors[4]
        elif stpc >= 6. and stpc < 8.:
            prob = .34
            color = self.alert_colors[4]
        elif stpc >= 8. and stpc < 10.:
            prob = .55
            color = self.alert_colors[5]
        elif stpc >= 10.:
            prob = .58
            color = self.alert_colors[5]
        else:
            prob = np.ma.masked
            color = self.alert_colors[0]
        return prob, color

    def stpf_prob(self, stpf):
        if stpf < .1:
            prob = .05
            color = self.alert_colors[0]
        elif stpf >= .1 and stpf < .5:
            prob = .06
            color = self.alert_colors[0]
        elif stpf >= .5 and stpf < 1.:
            prob = .11
            color = self.alert_colors[1]
        elif stpf >= 1. and stpf < 2.:
            prob = .17
            color = self.alert_colors[2]
        elif stpf >= 2. and stpf < 3.:
            prob = .25
            color = self.alert_colors[3]
        elif stpf >= 3. and stpf < 5.:
            prob = .25
            color = self.alert_colors[3]
        elif stpf >= 5. and stpf < 7.:
            prob = .39
            color = self.alert_colors[4]
        elif stpf >= 7. and stpf < 9.:
            prob = .55
            color = self.alert_colors[5]
        elif stpf >= 9.:
            prob = .59
            color = self.alert_colors[5]
        else:
            prob = np.ma.masked
            color = self.alert_colors[0]
        return prob, color


    def resizeEvent(self, e):
        '''
        Handles when the window is resized
        '''
        super(plotSTP, self).resizeEvent(e)
        self.plotData()
    
    def paintEvent(self, e):
        super(plotSTP, self).paintEvent(e)
        qp = QtGui.QPainter()
        qp.begin(self)
        qp.drawPixmap(1, 1, self.plotBitMap)
        qp.end()

    def clearData(self):
        '''
        Handles the clearing of the pixmap
        in the frame.
        '''
        self.plotBitMap = QtGui.QPixmap(self.width(), self.height())
        self.plotBitMap.fill(self.bg_color)
    
    def plotData(self):
        '''
        Handles painting on the frame
        '''
        if self.prof is None:
            return

        ## this function handles painting the plot
        ## create a new painter obkect
        qp = QtGui.QPainter()
        self.draw_stp(qp)
        self.draw_box(qp)

    def draw_stp(self, qp):
        if not tab.utils.QC(self.stpc):
            return

        qp.begin(self.plotBitMap)
        qp.setRenderHint(qp.Antialiasing)
        qp.setRenderHint(qp.TextAntialiasing)

        if self.stpc < 0:
            self.stpc = 0
        elif self.stpc > 11.:
            self.stpc = 11.
        prob, color = self.stpc_prob(self.stpc) 
        ef = self.stp_to_pix(self.stpc)
        pen = QtGui.QPen(color, 1.5, QtCore.Qt.SolidLine)
        qp.setPen(pen)
        qp.drawLine(0, ef, self.wid, ef)
        qp.end()

    def draw_box(self, qp):
        qp.begin(self.plotBitMap)
        width = self.brx / 14.
        left_x = width * 7
        right_x = self.brx - 5.
        top_y = self.stp_to_pix(11.)
        vspace = self.box_height + 1
        if platform.system() == "Windows":
            vspace += self.box_metrics.descent()
        bot_y = top_y + vspace * 8
        ## fill the box with a black background
        brush = QtGui.QBrush(self.bg_color, QtCore.Qt.SolidPattern)
        pen = QtGui.QPen(self.bg_color, 0, QtCore.Qt.SolidLine)
        qp.setPen(pen)
        qp.setBrush(brush)
        qp.drawRect(left_x, top_y, right_x - left_x, bot_y - top_y)
        ## draw the borders of the box
        pen = QtGui.QPen(self.fg_color, 2, QtCore.Qt.SolidLine)
        brush = QtGui.QBrush(QtCore.Qt.NoBrush)
        qp.setPen(pen)
        qp.setBrush(brush)
        qp.drawLine( left_x, top_y, right_x, top_y )
        qp.drawLine( left_x, bot_y, right_x, bot_y )
        qp.drawLine( left_x, top_y, left_x, bot_y )
        qp.drawLine( right_x, top_y, right_x, bot_y)
        ## set the font and line width for the rest of the plotting
        qp.setFont(self.box_font)
        ## plot the left column of text
        width = right_x - left_x - 3
        y1 = top_y + 2
        x1 = left_x+3
        x2 = x1 + (width * .75)
        ## start with the header/title
        texts = ['Prob EF2+ torn with supercell', 'Sample CLIMO = .15 sigtor']
        for text in texts:
            rect = QtCore.QRectF(x1, y1, width, self.box_height)
            qp.drawText(rect, QtCore.Qt.TextDontClip | QtCore.Qt.AlignLeft, text)
            vspace = self.box_height + 1
            if platform.system() == "Windows":
                vspace += self.box_metrics.descent()
            y1 += vspace
        vspace = y1 - 1
        if platform.system() == "Windows":
            vspace += 2
        qp.drawLine(left_x, vspace, right_x, vspace)
        ## draw the variable names
        texts = ['based on CAPE: ', 'based on LCL:', 'based on ESRH:', 'based on EBWD:',
                 'based on STPC:', 'based on STP_fixed:' ]
        probs = [self.cape_p, self.lcl_p, self.esrh_p, self.ebwd_p, self.stpc_p, self.stpf_p]
        colors = [self.cape_c, self.lcl_c, self.esrh_c, self.ebwd_c, self.stpc_c, self.stpf_c]
        for text, p, c in zip(texts, probs, colors):
            pen = QtGui.QPen(c, 1, QtCore.Qt.SolidLine)
            qp.setPen(pen)
            rect = QtCore.QRectF(x1, y1, width, self.box_height)
            rect2 = QtCore.QRectF(x2, y1, width, self.box_height)
            qp.drawText(rect, QtCore.Qt.TextDontClip | QtCore.Qt.AlignLeft, text)
            qp.drawText(rect2, QtCore.Qt.TextDontClip | QtCore.Qt.AlignLeft, tab.utils.FLOAT2STR(p,2) )
            vspace = self.box_height
            if platform.system() == "Windows":
                vspace += self.box_metrics.descent()
            y1 += vspace
        qp.end()

if __name__ == '__main__':
    app_frame = QtGui.QApplication([])        
    tester = plotSTP()
    tester.setGeometry(50,50,293,195)
    #tester.plotBitMap.save('test.png', format='png')
    tester.show()        
    app_frame.exec_()



import sys
import numpy as np
from qtpy import QtGui, QtCore, QtWidgets
from .. viz import plotSkewT, plotHodo, plotText, plotAnalogues
from .. viz import plotThetae, plotWinds, plotSpeed, plotKinematics
from .. viz import plotSlinky, plotWatch, plotAdvection, plotSTP
from .. viz import plotGeneric
from . constants import *
from . sounding import prof, plot_title


# Setup Application
app = QtGui.QApplication(sys.argv)
mainWindow = QtGui.QMainWindow()
x = 1180; y = 800
mainWindow.setGeometry(0, 0, x, y)
title = 'SHARPpy: Sounding and Hodograph Analysis and Research Program '
title += 'in Python'
mainWindow.setWindowTitle(title)
mainWindow.setStyleSheet("QMainWindow {background-color: rgb(0, 0, 0);}")
centralWidget = QtWidgets.QFrame()
mainWindow.setCentralWidget(centralWidget)
grid = QtGui.QGridLayout()
grid.setHorizontalSpacing(0)
grid.setVerticalSpacing(2)
centralWidget.setLayout(grid)


# Handle the Upper Left
## plot the main sounding
#print prof.right_scp, prof.left_scp
brand = 'SHARPpy Beta'
sound = plotSkewT(prof, pcl=prof.mupcl, title=plot_title, brand=brand)
sound.setContentsMargins(0, 0, 0, 0)
grid.addWidget(sound, 0, 0, 3, 1)

# Handle the Upper Right
urparent = QtWidgets.QFrame()
urparent_grid = QtGui.QGridLayout()
urparent_grid.setContentsMargins(0, 0, 0, 0)
urparent.setLayout(urparent_grid)
ur = QtWidgets.QFrame()
ur.setStyleSheet("QFrame {"
                 "  background-color: rgb(0, 0, 0);"
                 "  border-width: 0px;"
                 "  border-style: solid;"
                 "  border-color: rgb(255, 255, 255);"
                 "  margin: 0px;}")
brand = QtGui.QLabel('SHARPpy Beta')
brand.setAlignment(QtCore.Qt.AlignRight)
brand.setStyleSheet("QFrame {"
                    "  background-color: rgb(0, 0, 0);"
                    "  text-align: right;"
                    "  font-size: 11px;"
                    "  color: #FFFFFF;}")
grid2 = QtGui.QGridLayout()
grid2.setHorizontalSpacing(0)
grid2.setVerticalSpacing(0)
grid2.setContentsMargins(0, 0, 0, 0)
ur.setLayout(grid2)

speed_vs_height = plotSpeed( prof )
speed_vs_height.setObjectName("svh")
inferred_temp_advection = plotAdvection(prof)
hodo = plotHodo(prof.hght, prof.u, prof.v, prof=prof, centered=prof.mean_lcl_el)
storm_slinky = plotSlinky(prof)
#thetae_vs_pressure = plotThetae(prof)
thetae_vs_pressure = plotGeneric(prof.thetae[prof.pres > 500.], prof.pres[prof.pres > 500.],
                                 xticks=np.arange(320,360,10), yticks=np.arange(500, 1000, 100) )
srwinds_vs_height = plotWinds(prof)
#srwinds_vs_height = plotGeneric(prof.srwind, prof.hght)
watch_type = plotWatch(prof)

grid2.addWidget(speed_vs_height, 0, 0, 11, 3)
grid2.addWidget(inferred_temp_advection, 0, 3, 11, 2)
grid2.addWidget(hodo, 0, 5, 8, 24)
grid2.addWidget(storm_slinky, 8, 5, 3, 6)
grid2.addWidget(thetae_vs_pressure, 8, 11, 3, 6)
grid2.addWidget(srwinds_vs_height, 8, 17, 3, 6)
grid2.addWidget(watch_type, 8, 23, 3, 6)
urparent_grid.addWidget(brand, 0, 0, 1, 0)
urparent_grid.addWidget(ur, 1, 0, 50, 0)
grid.addWidget(urparent, 0, 1, 3, 1)

# Handle the Text Areas
text = QtWidgets.QFrame()
text.setStyleSheet("QWidget {"
                   "  background-color: rgb(0, 0, 0);"
                   "  border-width: 2px;"
                   "  border-style: solid;"
                   "  border-color: #3399CC;}")
grid3 = QtGui.QGridLayout()
grid3.setHorizontalSpacing(0)
grid3.setContentsMargins(0, 0, 0, 0)
text.setLayout(grid3)
convective = plotText(prof)
#convective = QtWidgets.QFrame()
#kinematic = QtWidgets.QFrame()
kinematic = plotKinematics(prof)
SARS = plotAnalogues(prof)
stp = plotSTP(prof)
grid3.addWidget(convective, 0, 0)
grid3.addWidget(kinematic, 0, 1)
grid3.addWidget(SARS, 0, 2)
grid3.addWidget(stp, 0, 3)
grid.addWidget(text, 3, 0, 1, 2)
pixmap = QtGui.QPixmap.grabWidget(mainWindow)
pixmap.save('skewt.png', 'PNG', 100)
mainWindow.show()

app.exec_()



import numpy as np
from . import thermo

__all__ = ['raiseError', 'numMasked', 'isPRESValid', 'isHGHTValid', 'isWSPDValid', 'areProfileArrayLengthEqual', 'raiseError', 'DataQualityException']
__all__ += ['isDWPCValid', 'isTMPCValid']

# Data quality exception error
class DataQualityException(Exception):
    pass

def raiseError(string, errorType):
    '''
        raiseError

        This function will raise an exception specified by the errorType variable.

        Parameters
        ----------
        string : str
            the string to be printed out to the user when the exception is thrown

        errorType : Exception
            the type of exception.

        Returns
        -------
        None

    '''
    raise Exception(errorType, string)

def numMasked(arr):
    '''
        numMasked

        This function returns the number of unmasked array elements as well as the
        length of the array.

        Parameters
        ----------
        arr : masked numpy array
            The maksed numpy array

        Returns
        -------
        count : number
            the number of unmasked array elements
        length : number
            the length of the unmasked array

    '''
    return arr.count(), arr.shape[0]

def areProfileArrayLengthEqual(prof):
    '''
        areProfileArrayLengthEqual

        This function checks to see if all of the arrays passed to a Profile object
        have the same length.  If they don't have the same length, then this function
        raises an AssertionError exception.

        Parameters
        ----------
        prof : profile object
            Profile object

        Returns
        -------
        None

    '''

    if not (len(prof.pres) == len(prof.hght) == len(prof.tmpc) == len(prof.dwpc) ==\
            len(prof.wdir) == len(prof.wspd) == len(prof.u) == len(prof.v) == len(prof.omeg)):
        raiseError("Arrays passed to the Profile object have unequal lengths.", DataQualityException)
 
def isPRESValid(pres):
    '''
        isPRESValid

        This function checks to see if valid pressure (mb)
        values are within the pres array that is passed to the 
        Profile object

        True If:
            1.) pressure array length is > 1
            2.) pressure array is not filled with masked values
            3.) if the pressure array is decreasing with the index
                and there are no repeat values.

        Parameters
        ----------
        pres : array
            pressure array (mb)

        Returns
        -------
        value : bool

    '''
    idx_diff = np.ma.diff(pres)
    neg_pres = np.ma.where(pres <= 0)[0]
    num_ok, total = numMasked(pres)
    if np.all(idx_diff < 0) and len(neg_pres) == 0 and num_ok > 1:
        return True
    else:
        return False

def isHGHTValid(hght):
    '''
        isHGHTValid

        This function checks to see if valid height (m)
        values are within the hght array that is passed to the 
        Profile object

        True/False: True If:
             1.) height array length is > 0
             2.) height array is not filled with masked values
             3.) if the height array is increasing with the index
                 and there are no repeat values.

        Parameters
        ----------
        hght : array
            height array (m)

        Returns
        -------
        value : bool

    '''
    num_ok, total = numMasked(hght)

    try:
        hght = hght[~hght.mask]
    except:
        pass

    idx_diff = np.ma.diff(hght)

    if np.all(idx_diff > 0) and num_ok > 1:
        return True
    else:
        return False

def isWDIRValid(wdir):
    '''
        isWDIRValid

        This function checks to see if valid wind direction (deg)
        values are within the wdir array that is passed to the 
        Profile object

        True if the wind direction array is > 0 in size and
        if the wind directions are between 0 and 360.

        Parameters
        ----------
        wdir : array
            wind direction array (degrees)

        Returns
        -------
        value : bool
    '''
    idx = np.ma.where((wdir > 360) | (wdir < 0))[0]
    if len(idx) == 0:
        return True
    else:
        return False

def isWSPDValid(wspd):
    '''
        isWSPDValid

        This function checks to see if valid wind speed (knots)
        values are within the wspd array that is passed to the 
        Profile object

        Parameters
        ----------
        wspd: the wind speed array (knots)

        Returns
        -------
        True/False: True if the wind speeds in the wspd array are all >= 0.
    '''
    idx = np.ma.where(wspd < 0)[0]
    if len(idx) == 0:
        return True
    else:
        return False

def isTMPCValid(tmpc):
    '''
        isTMPCValid

        This function checks to see if valid temperature (Celsius)
        values are within the tmpc array that is passed to the 
        Profile object

        Parameters
        ----------
        tmpc: the temperature array (Celsius)

        Returns
        -------
        True/False: True if the values in the temperature array are all above absolute zero.
    '''
    idx = np.ma.where(thermo.ctok(tmpc) <= 0)[0]
    if len(idx) == 0:
        return True
    else:
        return False

def isDWPCValid(dwpc):
    '''
        isDWPCValid

        This function checks to see if valid dewpoint (Celsius)
        values are within the dwpc array that is passed to the 
        Profile object

        Parameters
        ----------
        dwpc: the dewpoint array (Celsius)

        Returns
        -------
        True/False: True if the values in the dewpoint array are all above absolute zero.
    '''
    idx = np.ma.where(thermo.ctok(dwpc) <= 0)[0]
    if len(idx) == 0:
        return True
    else:
        return False





import numpy as np
import os
#import sharppy.io.spc_decoder as spc_decoder

## original database and code provided by
## Ryan Jewell - NOAA Storm Prediction Center (hail)
## Rich Thompson - NOAA Storm Prediction Center (supercell)

## Routines implemented in Python by Greg Blumberg - CIMMS and Kelton Halbert (OU SoM)
## wblumberg@ou.edu, greg.blumberg@noaa.gov, kelton.halbert@noaa.gov, keltonhalbert@ou.edu

def supercell(database_fn, mlcape, mllcl, h5temp, lr, shr, srh, shr3k, shr9k, srh3):
    '''
    The SARS Supercell database was provided by Rich Thompson of the 
    NOAA Storm Prediction Center in Norman, Oklahoma. This is a database
    of observed and model proximity soundings to both tornadic and 
    nontornadic supercell thunderstorms.
    
    This function works by searching for matches to particular variables
    in the database that have been found to be useful in forecasting 
    supercell thunderstorms. It searches through the database to find 
    matches to a given sounding within a range of uncertainty that
    has been tested internally in SPC to provide the best analogues
    possible. There is a loose criteria that is used to get probabilities,
    and there is a tighter criteria for quality matches that are supposed to
    be similar to the given sounding.
    
    The loose matches are based on the mixed layer cape (mlcape), mixed layer
    lcl (mllcl), 0-1km shear (shr), 0-1km storm relative helicity (srh), 500mb
    temperature (h5temp), and the 700-500mb lapse rate (lr).
    
    The ranges for the loose matches are set as such:
        mlcape: +/- 1300 J/kg
        mllcl: +/- 500 m
        shr: +/- 14 kts
        srh < 50 m^2/s^2: +/- 100 m^2/s^2
        srh >= 50 m^2/s^2: +/- 100% m^2/s^2
        h5temp: +/- 7.0 C
        lr: +/-1.0 C/km
    
    The quality matches are based on the same variables as the loose matches,
    with the addition of the 0-3km shear (shr3k), 0-9km shear (shr9k), and the
    0-3km Storm Relative Helicity (srh3).
    
    The ranges for the quality matches are set as such:
        mlcape: +/- 25% J/kg
        mllcl: +/- 200 m
        shr: +/- 10 kts
        srh < 100 m^2/s^2: +/- 50 m^2/s^2
        srh >= 100 m^2/s^2: +/- 30% m^2/s^2
        srh3 < 100 m^2/s^2: +/- 50 m^2/s^2
        srh3 >= 100 m^2/s^2: +/- 50% m^s/s^2
        h5temp: +/- 5 C
        lr: +/- 0.8 C/km
        shr3k: +/- 15 kts
        shr9k: +/- 25 kts
        
    Parameters
    ----------
    database_fn - filename of the database
    mlcape - the mixed layer cape (J/kg)
    mllcl - the mixed layer LCL (m)
    h5temp - the 500mb temp (C)
    lr - the 700-500mb lapse rate (C/km)
    shr - the 0-1km shear (kts)
    srh - the 0-1km storm relative helicity (m^2/s^2)
    shr3k - the 0-3km shear (kts)
    shr9k - the 0-9km shear (kts)
    srh3 - the 0-3km storm relative helicity (m^2/s^2)
    
    Returns
    -------
    quality_match_soundings: The dates/locations of the quality matches
    quality_match_tortype: The type of quality match (SIGTOR/WEAKTOR/NONTOR)
    len(loose_match_idx):  The number of loose matches
    num_matches: The number of weak and sig matches in the loose matches
    tor_prob: SARS sig. tornado probability
    '''
    # Open and read the file
    database_fn = os.path.join( os.path.dirname( __file__ ), database_fn)
    supercell_database = np.loadtxt(database_fn, skiprows=1, dtype=bytes, comments="%%%%")

    # Set range citeria for matching soundings
    # MLCAPE ranges
    if mlcape == 0:
        range_mlcape = 0
    else:
        range_mlcape = 1300 # J/kg
    range_mlcape_t1 = mlcape * 0.25 # J/kg

    # MLLCL ranges
    range_mllcl = 500 # m
    range_mllcl_t1 = 200 # m

    # 0-6 km shear ranges (kts)
    range_shr = 14
    range_shr_t1 = 10

    # 0-1 km SRH Ranges (m2/s2)
    if np.abs(srh) < 50:
        range_srh = 100.
    else:
        range_srh = srh

    if np.abs(srh) < 100:
        range_srh_t1 = 50
    else:
        range_srh_t1 = np.abs(srh) * 0.30

    # 0-3 SRH tier 1 ranges (m2/s2)
    if np.abs(srh3) < 100:
        range_srh3_t1 = 50.
    else:
        range_srh3_t1 = np.abs(srh3) * 0.50

    # 500 mb temperature ranges
    range_temp = 7 # C
    range_temp_t1 = 5 # C

    # 700-500 mb lapse rate ranges (C/km)
    range_lr = 1.0
    range_lr_t1 = 0.8

    # 3 km and 9 km shear matching
    range_shr3k_t1 = 15
    range_shr9k_t1 = 25
    ## Read in the columns for each variable
    mat_category = np.asarray(supercell_database[:,1], dtype=float) # category of match (0=non, 1=weak, 2=sig)
    mat_mlcape = np.asarray(supercell_database[:,3], dtype=float)
    mat_mllcl = np.asarray(supercell_database[:,5], dtype=float)
    mat_shr = np.asarray(supercell_database[:,7], dtype=float) # 0-6 KM SHEAR
    mat_srh = np.asarray(supercell_database[:,6], dtype=float) # 0-1 KM SRH
    mat_srh3 = np.asarray(supercell_database[:,14], dtype=float) # 0-3 KM SRH
    mat_h5temp = np.asarray(supercell_database[:,9], dtype=float) # 500 MB TEMP C
    mat_lr75 = np.asarray(supercell_database[:,11], dtype=float) # 700-500 MB LAPSE RATE
    mat_shr3 = np.asarray(supercell_database[:,12], dtype=float) # 0-3 KM SHEAR
    mat_shr9 = np.asarray(supercell_database[:,13], dtype=float) # 0-9 KM SHEAR
    ## Get the loose matches
    loose_match_idx = np.where((mlcape >= (mat_mlcape - range_mlcape)) & (mlcape <= (mat_mlcape + range_mlcape)) & \
                               (mllcl >= (mat_mllcl - range_mllcl)) & (mllcl <= (mat_mllcl + range_mllcl)) & \
                               (shr >= (mat_shr - range_shr)) & (shr <= (mat_shr + range_shr)) & \
                               (srh >= (mat_srh - range_srh)) & (srh <= (mat_srh + range_srh)) & \
                               (h5temp >= (mat_h5temp - range_temp)) & (h5temp <= (mat_h5temp + range_temp)) & \
                               (lr >= (mat_lr75 - range_lr)) & (lr <= (mat_lr75 + range_lr)))[0]

    num_matches = len(np.where(mat_category[loose_match_idx] > 0)[0]) #number of weak and sig matches in the loose matches

    ## Probability for tornado - needs to check to avoid zero division
    if len(loose_match_idx) > 0. and mlcape > 0:
        tor_prob = num_matches / float(len(loose_match_idx))
    else:
        tor_prob = 0.

    # Tier 1 matches (also known as the quality matches)
    quality_match_idx = np.where((mlcape >= (mat_mlcape - range_mlcape_t1)) & (mlcape <= (mat_mlcape + range_mlcape_t1)) & \
                               (mllcl >= (mat_mllcl - range_mllcl_t1)) & (mllcl <= (mat_mllcl + range_mllcl_t1)) & \
                               (shr >= (mat_shr - range_shr_t1)) & (shr <= (mat_shr + range_shr_t1)) & \
                               (srh >= (mat_srh - range_srh_t1)) & (srh <= (mat_srh + range_srh_t1)) & \
                               (h5temp >= (mat_h5temp - range_temp_t1)) & (h5temp <= (mat_h5temp + range_temp_t1)) & \
                               (lr >= (mat_lr75 - range_lr_t1)) & (lr <= (mat_lr75 + range_lr_t1)) & \
                               (shr3k >= (mat_shr3 - range_shr3k_t1)) & (shr3k <= (mat_shr3 + range_shr3k_t1)) & \
                               (shr9k >= (mat_shr9 - range_shr9k_t1)) & (shr9k <= (mat_shr9 + range_shr9k_t1)) & \
                               (srh3 >= (mat_srh3 - range_srh3_t1)) & (srh3 <= (mat_srh3 + range_srh3_t1)))[0]

    quality_match_soundings = supercell_database[:,0][quality_match_idx]
    quality_match_tortype = np.asarray(supercell_database[:,1][quality_match_idx], dtype='|S7')

    np.place(quality_match_tortype, quality_match_tortype==b'2', 'SIGTOR')
    np.place(quality_match_tortype, quality_match_tortype==b'1', 'WEAKTOR')
    np.place(quality_match_tortype, quality_match_tortype==b'0', 'NONTOR')

    quality_match_soundings = np.array([ qms.decode('utf-8') for qms in quality_match_soundings ])
    quality_match_tortype = np.array([ qmt.decode('utf-8') for qmt in quality_match_tortype ])

    return quality_match_soundings, quality_match_tortype, len(loose_match_idx), num_matches, tor_prob





def hail(database_fn, mumr, mucape, h5_temp, lr, shr6, shr9, shr3, srh):
    '''
    The SARS Hail database was provided by Ryan Jewell of the NOAA Storm
    Prediction Center in Norman, Oklahoma. This is a database of observed
    and model proximity analogue soundings to hail events.
    
    This function works by searching for matches to particular variables in 
    the database that have been attributed to hail events given a certain 
    range of uncertainty. The loose matches are used for statistics such as
    % significant hail matches, and are based on a looser criteria for
    matches. The analogues that get displayed are based on a tighter criteria
    for matches to insure that only quality matches are received. Ranges for
    loose and quality matches are semi-arbitrary, and were tuned by testing
    internally by SPC.
    
    The loose matches are based on the most unstable parcel mixing ratio (mumr),
    most unstable cape (mucape), 700-500mb labse rate (lr), 500mb temperature
    (h5_temp), 0-3km shear (shr3), 0-6km shear (shr6), and the 0-9km shear (shr9).
    
    The ranges for the loose matches are set as such:
        mumr: +/- 2.0 g/kg
        mucape: +/- 30% J/kg
        lr: +/- 2.0 C/km
        h5_temp: +/- 9.0 C
        shr6: +/- 12 m^2/s^2
        shr9: +/- 22 m^2/s^2
        shr3: +/- 10 m^2/s^2
    
    The quality matches use the fields described in the loose matches, plus the
    0-3km Storm Relative Helicity (srh). The bounds for the search for quality 
    matches is much more strict that the loose matches.
    
    The ranges for the quality matches are set as such:
        mumr: +/- 2.0 g/kg
        mucape < 500 J/kg: +/- 50% J/kg
        mucape < 2000 J/kg: +/- 25% J/kg
        mucape >= 2000 J/kg: +/- 20% J/kg
        lr: +/- 0.4 C/km
        h5_temp: +/- 1.5 C/km
        shr6: +/- 6 m^2/s^2
        shr9: +/- 15 m^2/s^2
        shr3: +/- 8 m^2/s^2
        srh < 50 m^2/s^2: +/- 25 m^2/s^2
        srh >= 50 m^2/s^2: +/- 50% m^2/s^2
    
    Parameters
    ----------
    mumr - most unstable parcel mixing ratio (g/kg)
    mucape - most unstable CAPE (J/kg)
    h5_temp - 500 mb temperature (C)
    lr - 700-500 mb lapse rate (C/km)
    shr6 - 0-6 km shear (m/s)
    shr9 - 0-9 km shear (m/s)
    shr3 - 0-3 km shear (m/s)
    srh - 0-3 Storm Relative Helicity (m2/s2)
    
    Returns
    -------
    quality_match_dates (str) - dates of the quality matches
    quality_match_sizes (float) - hail sizes of the quality matches
    num_loose_matches (int) - number of loose matches
    num_sig_reports (int) - number of significant hail reports (>= 2 inches)
    prob_sig_hail (float) - SARS sig. hail probability
    
    '''
    ## open the file in the current directory with the name database_fn
    database_fn = os.path.join( os.path.dirname( __file__ ), database_fn )
    hail_database = np.loadtxt(database_fn, skiprows=1, dtype=bytes)

    #Set range criteria for matching sounding
    # MU Mixing Ratio Ranges
    range_mumr = 2.0 # g/kg
    range_mumr_t1 = 2.0 # g/kg

    # MUCAPE Ranges (J/kg)
    range_mucape = mucape*.30
    if mucape < 500.:
        range_mucape_t1 = mucape * .50
    elif mucape >= 500. and mucape < 2000.:
        range_mucape_t1 = mucape * .25
    else:
        range_mucape_t1 = mucape * .20

    # 700-500 mb Lapse Rate Ranges
    range_lr = 2.0 # C/km
    range_lr_t1 = 0.4 # C/km

    # 500 mb temperature ranges
    range_temp = 9 # C
    range_temp_t1 = 1.5 # C

    # 0-6 km shear ranges
    range_shr6 = 12 # m/s
    range_shr6_t1 = 6 # m/s

    # 0-9 km shear ranges

    range_shr9 = 22 # m/s
    range_shr9_t1 = 15 # m/s

    # 0-3 km shear ranges
    range_shr3 = 10
    range_shr3_t1 = 8

    # 0-3 SRH Ranges
    range_srh = 100
    if srh < 50:
        range_srh_t1 = 25
    else:
        range_srh_t1 = srh * 0.5

    #Get database variables from the columns in the file and make them floats
    matmr = np.asarray(hail_database[:,4], dtype=float) # MU Mixing Ratio
    matcape = np.asarray(hail_database[:,3], dtype=float) # MUCAPE
    matlr = np.asarray(hail_database[:,7], dtype=float) # 700-500 mb lapse rate
    mattemp = np.asarray(hail_database[:,5], dtype=float) # 500 mb temp
    matshr6 = np.asarray(hail_database[:,10], dtype=float) # 0-6 shear
    matshr9 = np.asarray(hail_database[:,11], dtype=float) # 0-9 shear
    matshr3 = np.asarray(hail_database[:,9], dtype=float) # 0-3 shear
    matsrh = np.asarray(hail_database[:,12], dtype=float) # 0-3 SRH

    # Find the loose matches using the ranges set above
    loose_match_idx = np.where((mumr >= (matmr - range_mumr)) & (mumr <= (matmr + range_mumr)) & \
                               (mucape >= (matcape - range_mucape)) & (mucape <= (matcape + range_mucape)) & \
                               (lr >= (matlr - range_lr)) & (lr <= (matlr + range_lr)) & \
                               (h5_temp >= (mattemp - range_temp)) & (h5_temp <= (mattemp + range_temp)) & \
                               (shr6 >= (matshr6 - range_shr6)) & (shr6 <= (matshr6 + range_shr6)) & \
                               (shr9 >= (matshr9 - range_shr9)) & (shr9 <= (matshr9 + range_shr9)) & \
                               (shr3 >= (matshr3 - range_shr3)) & (shr3 <= (matshr3 + range_shr3)))[0]
    ## How many loose matches are there?
    num_loose_matches = float(len(loose_match_idx))
    ## What were the sizes of those matches?
    hail_sizes = np.asarray(hail_database[:,2], dtype=float)
    ## How many of them were significant (>2.0 in)?
    num_sig_reports = float(len(np.where(hail_sizes[loose_match_idx] >= 2.)[0]))

    ## Calculate the Probability of significant hail - must make sure
    ## loose matches are > 0 to prevent division by 0.
    if num_loose_matches > 0 and mucape > 0:
        prob_sig_hail = num_sig_reports / num_loose_matches
        # Calculate the average hail size from the loose matches
        avg_hail_size = np.mean(hail_sizes[loose_match_idx])
    else:
        prob_sig_hail = 0

    # Find the quality matches
    quality_match_idx = np.where((mumr >= (matmr - range_mumr_t1)) & (mumr <= (matmr + range_mumr_t1)) & \
                               (mucape >= (matcape - range_mucape_t1)) & (mucape <= (matcape + range_mucape_t1)) & \
                               (lr >= (matlr - range_lr_t1)) & (lr <= (matlr + range_lr_t1)) & \
                               (h5_temp >= (mattemp - range_temp_t1)) & (h5_temp <= (mattemp + range_temp_t1)) & \
                               (shr6 >= (matshr6 - range_shr6_t1)) & (shr6 <= (matshr6 + range_shr6_t1)) & \
                               (shr9 >= (matshr9 - range_shr9_t1)) & (shr9 <= (matshr9 + range_shr9_t1)) & \
                               (shr3 >= (matshr3 - range_shr3_t1)) & (shr3 <= (matshr3 + range_shr3_t1)) & \
                               (srh >= (matsrh - range_srh_t1)) & (srh <= (matsrh + range_srh_t1)))[0]

    quality_match_dates = hail_database[quality_match_idx,0]
    quality_match_sizes = np.asarray(hail_database[quality_match_idx,2], dtype=float)

    # This filtering was in the sars.f file so the graphical output wasn't overrun by historical quality matches
    max_quality_matches = 15
    quality_match_dates = quality_match_dates[:max_quality_matches]
    quality_match_sizes = quality_match_sizes[:max_quality_matches]

    quality_match_dates = np.array([ qmd.decode('utf-8') for qmd in quality_match_dates ])

    return quality_match_dates, quality_match_sizes, num_loose_matches, num_sig_reports, prob_sig_hail


def get_sars_dir(match_type):
    """
    Returns the directory where the raw SARS files are.
    
    Parameters
    ----------
    match_type : str
        'supercell' or 'hail'
    Returns
    -------
    string

    """
    return os.path.join(os.path.dirname(__file__), "sars/" + match_type.lower() + "/")

## written by Kelton Halbert
def getSounding(match_string, match_type, profile="default"):
    """
    Given a match string and type (supercell:hail) from one of the
    SARS routines, decode the raw sounding data into a Profile
    object and return said object. It will default to a "default"
    profile object that does not compute any indices, but can be
    set to profile="convective" if indices are desired.

    :param match_string:
    :param match_type:
    :return: a Profile object
    """
    ## make sure the requested match type is valid
    #if (match_type.lower() != "supercell") or (match_type.lower() != "hail"):
    #    raise Exception("InvalidSARSType", match_type.lower() + " is an invalid SARS type.")
    ## get the directory with the data
    data_dir = get_sars_dir(match_type)

    match_date, match_loc = match_string.split(".")
    files = os.listdir(data_dir)
    for file in files:
        if file.startswith(match_date) and file.endswith(match_loc.lower()):
            datafile = data_dir + file
            break
        elif file.startswith(match_date) and file.endswith(match_loc.upper()):
            datafile = data_dir + file
            break
    return datafile




import logging
import numpy as np
from qtpy import QtGui, QtCore, QtWidgets
import sharppy.sharptab as tab
from sharppy.viz.barbs import drawBarb
from sharppy.sharptab.constants import *
import platform

## routine written by Kelton Halbert
## keltonhalbert@ou.edu

__all__ = ['backgroundKinematics', 'plotKinematics']

class backgroundKinematics(QtWidgets.QFrame):
    '''
    Handles drawing the background frame.
    '''
    def __init__(self, **kwargs):
        super(backgroundKinematics, self).__init__()
        self.wind_units = kwargs.get('wind_units', 'knots')
        self.initUI()

    def initUI(self):
        ## initialize fram variables such as size,
        ## padding, etc.
        self.setStyleSheet("QFrame {"
            "  background-color: rgb(0, 0, 0);"
            "  border-width: 1px;"
            "  border-style: solid;"
            "  border-color: #3399CC;}")

        self.lpad = 5; self.rpad = 5
        self.tpad = 5; self.bpad = 5

        self.wid = self.size().width()
        self.hgt = self.size().height()
        self.tlx = self.rpad; self.tly = self.tpad
        self.brx = self.wid; self.bry = self.hgt

        fsize = np.floor(.06 * self.hgt)
        self.tpad = np.floor(.03 * self.hgt)

        self.label_font = QtGui.QFont('Helvetica')
        self.label_font.setPixelSize(fsize)
        self.label_metrics = QtGui.QFontMetrics( self.label_font )
        self.label_height = self.label_metrics.xHeight() + self.tpad
        self.ylast = self.label_height
        self.barby = 0
        self.plotBitMap = QtGui.QPixmap(self.width()-2, self.height()-2)
        self.plotBitMap.fill(self.bg_color)
        self.plotBackground()
    
    def draw_frame(self, qp):
        '''
        Draws the background frame and the text headers for indices.
        '''
        ## initialize a white pen with thickness 1 and a solid line
        pen = QtGui.QPen(self.fg_color, 1, QtCore.Qt.SolidLine)
        qp.setPen(pen)
        qp.setFont(self.label_font)
        ## make the initial x value relative to the width of the frame
        x1 = self.brx / 10
        y1 = self.label_height + self.tpad
        ## draw the header
        rect1 = QtCore.QRect(x1*2.5, 3, x1, self.label_height)
        rect2 = QtCore.QRect(x1*5, 3, x1, self.label_height)
        rect3 = QtCore.QRect(x1*7, 3, x1, self.label_height)
        rect4 = QtCore.QRect(x1*9-self.rpad, 3, x1, self.label_height)

        if self.wind_units == 'm/s':
            disp_unit = 'm/s'
        else:
            disp_unit = 'kt'

        qp.drawText(rect1, QtCore.Qt.TextDontClip | QtCore.Qt.AlignCenter, 'SRH (m2/s2)')
        qp.drawText(rect2, QtCore.Qt.TextDontClip | QtCore.Qt.AlignCenter, "Shear (%s)" % disp_unit)
        qp.drawText(rect3, QtCore.Qt.TextDontClip | QtCore.Qt.AlignCenter, 'MnWind')
        qp.drawText(rect4, QtCore.Qt.TextDontClip | QtCore.Qt.AlignCenter, 'SRW')
        ## left column
        ## first block
        texts = ['SFC-1km', 'SFC-3km', 'Eff Inflow Layer',]
        for text in texts:
            rect = QtCore.QRect(self.lpad, y1, x1, self.label_height)
            qp.drawText(rect, QtCore.Qt.TextDontClip | QtCore.Qt.AlignLeft, text)
            vspace = self.label_height
            if platform.system() == "Windows":
                vspace += self.label_metrics.descent()
            y1 += vspace
        self.ylast = y1
        ## second block
        texts = ['SFC-6km', 'SFC-8km','LCL-EL (Cloud Layer)', 'Eff Shear (EBWD)']
        y1 = self.ylast + self.tpad
        for text in texts:
            rect = QtCore.QRect(self.lpad, y1, x1, self.label_height)
            qp.drawText(rect, QtCore.Qt.TextDontClip | QtCore.Qt.AlignLeft, text)
            vspace = self.label_height
            if platform.system() == "Windows":
                vspace += self.label_metrics.descent()
            y1 += vspace
        self.ylast = y1
        ## third block
        texts = ['BRN Shear = ', '4-6km SR Wind = ']
        y1 = self.ylast + self.tpad
        for text in texts:
            rect = QtCore.QRect(self.lpad, y1, x1, self.label_height)
            qp.drawText(rect, QtCore.Qt.TextDontClip | QtCore.Qt.AlignLeft, text)
            vspace = self.label_height 
            if platform.system() == "Windows":
                vspace += self.label_metrics.descent()
            y1 += vspace
        self.ylast = y1
        ## fourth block
        texts = ['...Storm Motion Vectors...', 'Bunkers Right = ', 'Bunkers Left = ', 'Corfidi Downshear = ', 'Corfidi Upshear = ']
        y1 = self.ylast + self.tpad
        self.barby = y1 + self.tpad
        for text in texts:
            rect = QtCore.QRect(self.lpad, y1, x1, self.label_height)
            qp.drawText(rect, QtCore.Qt.TextDontClip | QtCore.Qt.AlignLeft, text)
            vspace = self.label_height
            if platform.system() == "Windows":
                vspace += self.label_metrics.descent()
            y1 += vspace
        self.ylast = vspace

        ## draw lines seperating the indices
        qp.drawLine( 0, self.ylast+3, self.brx, self.ylast+3 )
    
    def resizeEvent(self, e):
        '''
        Handles when the window gets resized.
        '''
        self.initUI()

    def plotBackground(self):
        '''
        Handles drawing the text background.
        '''
        ## initialize a QPainter objext
        qp = QtGui.QPainter()
        qp.begin(self.plotBitMap)
        ## draw the frame
        self.draw_frame(qp)
        qp.end()


class plotKinematics(backgroundKinematics):
    '''
    Handles plotting the indices in the frame.
    '''
    def __init__(self, **kwargs):
        ## get the surfce based, most unstable, and mixed layer
        ## parcels to use for indices, as well as the sounding
        ## profile itself.

        self.bg_color = QtGui.QColor('#000000')
        self.fg_color = QtGui.QColor('#ffffff')
        self.use_left = False

        super(plotKinematics, self).__init__(**kwargs)
        self.prof = None

    def setProf(self, prof):
        self.ylast = self.label_height

        self.prof = prof

        if self.use_left:
            self.srh1km = prof.left_srh1km
            self.srh3km = prof.left_srh3km
            self.esrh = prof.left_esrh

            self.srw_1km = prof.left_srw_1km
            self.srw_3km = prof.left_srw_3km
            self.srw_6km = prof.left_srw_6km
            self.srw_8km = prof.left_srw_8km
            self.srw_lcl_el = prof.left_srw_lcl_el
            self.srw_4_5km = prof.left_srw_4_5km
            srw_eff = prof.left_srw_eff
            srw_ebw = prof.left_srw_ebw
        else:
            self.srh1km = prof.right_srh1km
            self.srh3km = prof.right_srh3km
            self.esrh = prof.right_esrh

            self.srw_1km = prof.right_srw_1km
            self.srw_3km = prof.right_srw_3km
            self.srw_6km = prof.right_srw_6km
            self.srw_8km = prof.right_srw_8km
            self.srw_lcl_el = prof.right_srw_lcl_el
            self.srw_4_5km = prof.right_srw_4_5km
            srw_eff = prof.right_srw_eff
            srw_ebw = prof.right_srw_ebw

        self.mean_1km = prof.mean_1km
        self.mean_3km = prof.mean_3km
        self.mean_6km = prof.mean_6km
        self.mean_8km = prof.mean_8km
        self.mean_lcl_el = prof.mean_lcl_el
        mean_eff = prof.mean_eff
        mean_ebw = prof.mean_ebw

        self.sfc_1km_shear = prof.sfc_1km_shear
        self.sfc_3km_shear = prof.sfc_3km_shear
        self.sfc_6km_shear = prof.sfc_6km_shear
        self.sfc_8km_shear = prof.sfc_8km_shear
        self.lcl_el_shear = prof.lcl_el_shear
        self.eff_shear = prof.eff_shear
        self.ebwd = prof.ebwd

        if prof.etop is np.ma.masked or prof.ebottom is np.ma.masked:
            self.mean_eff = [np.ma.masked, np.ma.masked]
            self.mean_ebw = [np.ma.masked, np.ma.masked]
            self.srw_eff = [np.ma.masked, np.ma.masked]
            self.srw_ebw = [np.ma.masked, np.ma.masked]
        else:
            self.mean_eff = tab.utils.comp2vec(mean_eff[0], mean_eff[1])
            self.mean_ebw = tab.utils.comp2vec(mean_ebw[0], mean_ebw[1])
            self.srw_eff = tab.utils.comp2vec(srw_eff[0], srw_eff[1])
            self.srw_ebw = tab.utils.comp2vec(srw_ebw[0], srw_ebw[1])

        self.brn_shear = prof.mupcl.brnshear
        self.bunkers_right_vec = tab.utils.comp2vec(prof.bunkers[0], prof.bunkers[1])
        self.bunkers_left_vec = tab.utils.comp2vec(prof.bunkers[2], prof.bunkers[3])
        self.upshear = tab.utils.comp2vec(prof.upshear_downshear[0],prof.upshear_downshear[1])
        self.downshear = tab.utils.comp2vec(prof.upshear_downshear[2],prof.upshear_downshear[3])

        self.clearData()
        self.plotBackground()
        self.plotData()
        self.update()

    def setPreferences(self, update_gui=True, **prefs):
        self.wind_units = prefs['wind_units']

        self.bg_color = QtGui.QColor(prefs['bg_color'])
        self.fg_color = QtGui.QColor(prefs['fg_color'])

        if update_gui:
            if self.use_left:
                self.srh1km = self.prof.left_srh1km
                self.srh3km = self.prof.left_srh3km
                self.esrh = self.prof.left_esrh

                self.srw_1km = self.prof.left_srw_1km
                self.srw_3km = self.prof.left_srw_3km
                self.srw_6km = self.prof.left_srw_6km
                self.srw_8km = self.prof.left_srw_8km
                self.srw_lcl_el = self.prof.left_srw_lcl_el
                self.srw_4_5km = self.prof.left_srw_4_5km
                srw_eff = self.prof.left_srw_eff
                srw_ebw = self.prof.left_srw_ebw
            else:
                self.srh1km = self.prof.right_srh1km
                self.srh3km = self.prof.right_srh3km
                self.esrh = self.prof.right_esrh

                self.srw_1km = self.prof.right_srw_1km
                self.srw_3km = self.prof.right_srw_3km
                self.srw_6km = self.prof.right_srw_6km
                self.srw_8km = self.prof.right_srw_8km
                self.srw_lcl_el = self.prof.right_srw_lcl_el
                self.srw_4_5km = self.prof.right_srw_4_5km
                srw_eff = self.prof.right_srw_eff
                srw_ebw = self.prof.right_srw_ebw

            if self.prof.etop is np.ma.masked or self.prof.ebottom is np.ma.masked:
                self.srw_eff = [np.ma.masked, np.ma.masked]
                self.srw_ebw = [np.ma.masked, np.ma.masked]
            else:
                self.srw_eff = tab.utils.comp2vec(srw_eff[0], srw_eff[1])
                self.srw_ebw = tab.utils.comp2vec(srw_ebw[0], srw_ebw[1])

            self.clearData()
            self.plotBackground()
            self.plotData()
            self.update()

    def setDeviant(self, deviant):
        self.use_left = deviant == 'left'

        if self.use_left:
            self.srh1km = self.prof.left_srh1km
            self.srh3km = self.prof.left_srh3km
            self.esrh = self.prof.left_esrh

            self.srw_1km = self.prof.left_srw_1km
            self.srw_3km = self.prof.left_srw_3km
            self.srw_6km = self.prof.left_srw_6km
            self.srw_8km = self.prof.left_srw_8km
            self.srw_lcl_el = self.prof.left_srw_lcl_el
            self.srw_4_5km = self.prof.left_srw_4_5km
            srw_eff = self.prof.left_srw_eff
            srw_ebw = self.prof.left_srw_ebw
        else:
            self.srh1km = self.prof.right_srh1km
            self.srh3km = self.prof.right_srh3km
            self.esrh = self.prof.right_esrh

            self.srw_1km = self.prof.right_srw_1km
            self.srw_3km = self.prof.right_srw_3km
            self.srw_6km = self.prof.right_srw_6km
            self.srw_8km = self.prof.right_srw_8km
            self.srw_lcl_el = self.prof.right_srw_lcl_el
            self.srw_4_5km = self.prof.right_srw_4_5km
            srw_eff = self.prof.right_srw_eff
            srw_ebw = self.prof.right_srw_ebw

        if self.prof.etop is np.ma.masked or self.prof.ebottom is np.ma.masked:
            self.srw_eff = [np.ma.masked, np.ma.masked]
            self.srw_ebw = [np.ma.masked, np.ma.masked]
        else:
            self.srw_eff = tab.utils.comp2vec(srw_eff[0], srw_eff[1])
            self.srw_ebw = tab.utils.comp2vec(srw_ebw[0], srw_ebw[1])

        self.clearData()
        self.plotBackground()
        self.plotData()
        self.update()

    def resizeEvent(self, e):
        '''
        Handles when the window is resized.
        '''
        super(plotKinematics, self).resizeEvent(e)
        self.plotData()
    
    def paintEvent(self, e):
        super(plotKinematics, self).paintEvent(e)
        qp = QtGui.QPainter()
        qp.begin(self)
        qp.drawPixmap(1, 1, self.plotBitMap)
        qp.end()

    def clearData(self):
        '''
        Handles the clearing of the pixmap
        in the frame.
        '''
        self.plotBitMap = QtGui.QPixmap(self.width(), self.height())
        self.plotBitMap.fill(self.bg_color)

    def plotData(self):
        '''
        Handles the drawing of the text on the frame.
        '''
        if self.prof is None:
            return

        x1 = self.brx / 10
        y1 = self.bry / 19
        origin_x = x1*8.5
        origin_y = y1*15

        ## initialize a QPainter object
        qp = QtGui.QPainter()
        qp.begin(self.plotBitMap)
        qp.setRenderHint(qp.Antialiasing)
        qp.setRenderHint(qp.TextAntialiasing)
        ## draw the indices
        self.drawKinematics(qp)
        self.drawBarbs(qp)
        qp.end()
    
    def drawBarbs(self, qp):
        x1 = self.brx / 10
        y1 = self.bry / 19
        origin_x = x1*8.
        pen = QtGui.QPen(QtGui.QColor('#0A74C6'), 1, QtCore.Qt.SolidLine)
        qp.setPen(pen)
        qp.setFont(self.label_font)
        rect0 = QtCore.QRect(x1*7, self.ylast + self.tpad, x1*2, self.label_height)
        qp.drawText(rect0, QtCore.Qt.TextDontClip | QtCore.Qt.AlignCenter, '1km & 6km AGL\nWind Barbs' )

        if self.wind_units == "m/s":
            conv = tab.utils.KTS2MS
        else:
            conv = lambda s: s
        try:
            drawBarb(qp, origin_x, self.barby, self.prof.wind1km[0], conv(self.prof.wind1km[1]), color='#AA0000', shemis=(self.prof.latitude < 0))
            drawBarb(qp, origin_x, self.barby, self.prof.wind6km[0], conv(self.prof.wind6km[1]), color='#0A74C6', shemis=(self.prof.latitude < 0))
        except:
            logging.debug("Couldn't draw the 1 or 6 km wind barbs on kinematics.py")
 
    def drawKinematics(self, qp):
        '''
        This handles the severe indices, such as STP, sig hail, etc.
        ---------
        qp: QtGui.QPainter object
        '''
        ## initialize a pen to draw with.
        pen = QtGui.QPen(self.fg_color, 1, QtCore.Qt.SolidLine)
        qp.setPen(pen)
        qp.setFont(self.label_font)
        x1 = self.brx / 10
        y1 = self.ylast + self.tpad

        if self.wind_units == 'm/s':
            disp_unit = ' m/s'
            conv = tab.utils.KTS2MS
        else:
            disp_unit = ' kt'
            conv = lambda s: s

        ## format the text
        srh1km = tab.utils.INT2STR(self.srh1km[0])
        srh3km = tab.utils.INT2STR(self.srh3km[0])
        
        sfc1km = tab.utils.INT2STR(conv(tab.utils.mag(self.sfc_1km_shear[0], self.sfc_1km_shear[1])))
        sfc3km = tab.utils.INT2STR(conv(tab.utils.mag(self.sfc_3km_shear[0], self.sfc_3km_shear[1])))
        sfc6km = tab.utils.INT2STR(conv(tab.utils.mag(self.sfc_6km_shear[0], self.sfc_6km_shear[1])))
        sfc8km = tab.utils.INT2STR(conv(tab.utils.mag(self.sfc_8km_shear[0], self.sfc_8km_shear[1])))
        lcl_el = tab.utils.INT2STR(conv(tab.utils.mag(self.lcl_el_shear[0], self.lcl_el_shear[1])))
        
        mean_1km = tab.utils.INT2STR(np.float64(self.mean_1km[0])) + '/' + tab.utils.INT2STR(conv(self.mean_1km[1]))
        mean_3km = tab.utils.INT2STR(np.float64(self.mean_3km[0])) + '/' + tab.utils.INT2STR(conv(self.mean_3km[1]))
        mean_6km = tab.utils.INT2STR(np.float64(self.mean_6km[0])) + '/' + tab.utils.INT2STR(conv(self.mean_6km[1]))
        mean_8km = tab.utils.INT2STR(np.float64(self.mean_8km[0])) + '/' + tab.utils.INT2STR(conv(self.mean_8km[1]))
        mean_lcl_el = tab.utils.INT2STR(np.float64(self.mean_lcl_el[0])) + '/' + tab.utils.INT2STR(conv(self.mean_lcl_el[1]))
        
        srw_1km = tab.utils.INT2STR(np.float64(self.srw_1km[0])) + '/' + tab.utils.INT2STR(conv(self.srw_1km[1]))
        srw_3km = tab.utils.INT2STR(np.float64(self.srw_3km[0])) + '/' + tab.utils.INT2STR(conv(self.srw_3km[1]))
        srw_6km = tab.utils.INT2STR(np.float64(self.srw_6km[0])) + '/' + tab.utils.INT2STR(conv(self.srw_6km[1]))
        srw_8km = tab.utils.INT2STR(np.float64(self.srw_8km[0])) + '/' + tab.utils.INT2STR(conv(self.srw_8km[1]))
        srw_lcl_el = tab.utils.INT2STR(np.float64(self.srw_lcl_el[0])) + '/' + tab.utils.INT2STR(conv(self.srw_lcl_el[1]))
        srw_4_5km = tab.utils.INT2STR(np.float64(self.srw_4_5km[0])) + '/' + tab.utils.INT2STR(conv(self.srw_4_5km[1])) + disp_unit
        
        esrh = tab.utils.INT2STR(self.esrh[0])
        eff_lr = tab.utils.INT2STR(conv(tab.utils.mag(self.eff_shear[0], self.eff_shear[1])))
        efbwd = tab.utils.INT2STR(conv(tab.utils.mag(self.ebwd[0], self.ebwd[1])))
        mean_eff = tab.utils.INT2STR(np.float64(self.mean_eff[0])) + '/' + tab.utils.INT2STR(conv(self.mean_eff[1]))
        mean_ebw = tab.utils.INT2STR(np.float64(self.mean_ebw[0])) + '/' + tab.utils.INT2STR(conv(self.mean_ebw[1]))
        srw_eff = tab.utils.INT2STR(np.float64(self.srw_eff[0])) + '/' + tab.utils.INT2STR(conv(self.srw_eff[1]))
        srw_ebw = tab.utils.INT2STR(np.float64(self.srw_ebw[0])) + '/' + tab.utils.INT2STR(conv(self.srw_ebw[1]))
        
        brn_shear = tab.utils.INT2STR(self.brn_shear) + ' m2/s2'
        bunkers_left = tab.utils.INT2STR(np.float64(self.bunkers_left_vec[0])) + '/' + tab.utils.INT2STR(conv(self.bunkers_left_vec[1])) + disp_unit
        bunkers_right = tab.utils.INT2STR(np.float64(self.bunkers_right_vec[0])) + '/' + tab.utils.INT2STR(conv(self.bunkers_right_vec[1])) + disp_unit
        upshear = tab.utils.INT2STR(np.float64(self.upshear[0])) + '/' + tab.utils.INT2STR(conv(self.upshear[1])) + disp_unit
        downshear = tab.utils.INT2STR(np.float64(self.downshear[0])) + '/' + tab.utils.INT2STR(conv(self.downshear[1])) + disp_unit

        ## sfc-1km
        texts = [srh1km, sfc1km, mean_1km, srw_1km]
        count = 3
        for text in texts:
            rect = QtCore.QRect(x1*count, y1, x1, self.label_height)
            qp.drawText(rect, QtCore.Qt.TextDontClip | QtCore.Qt.AlignRight, text)
            count += 2
        vspace = self.label_height
        if platform.system() == "Windows":
            vspace += self.label_metrics.descent()
        y1 += vspace
        self.ylast = y1
        ## sfc-3km
        texts = [srh3km, sfc3km, mean_3km, srw_3km]
        count = 3
        for text in texts:
            rect = QtCore.QRect(x1*count, y1, x1, self.label_height)
            qp.drawText(rect, QtCore.Qt.TextDontClip | QtCore.Qt.AlignRight, text)
            count += 2
        vspace = self.label_height
        if platform.system() == "Windows":
            vspace += self.label_metrics.descent()
        y1 += vspace
        self.ylast = y1
        ## Effective Inflow Layer
        texts = [esrh, eff_lr, mean_eff, srw_eff]
        count = 3
        for text in texts:
            rect = QtCore.QRect(x1*count, y1, x1, self.label_height)
            qp.drawText(rect, QtCore.Qt.TextDontClip | QtCore.Qt.AlignRight, text)
            count += 2
        vspace = self.label_height + self.tpad
        if platform.system() == "Windows":
            vspace += self.label_metrics.descent()
        y1 += vspace
        self.ylast = y1
        ## sfc-6km
        texts = [sfc6km, mean_6km, srw_6km]
        count = 5
        for text in texts:
            rect = QtCore.QRect(x1*count, y1, x1, self.label_height)
            qp.drawText(rect, QtCore.Qt.TextDontClip | QtCore.Qt.AlignRight, text)
            count += 2
        vspace = self.label_height
        if platform.system() == "Windows":
            vspace += self.label_metrics.descent()
        y1 += vspace
        self.ylast = y1
        ## sfc-8km
        texts = [sfc8km, mean_8km, srw_8km]
        count = 5
        for text in texts:
            rect = QtCore.QRect(x1*count, y1, x1, self.label_height)
            qp.drawText(rect, QtCore.Qt.TextDontClip | QtCore.Qt.AlignRight, text)
            count += 2
        vspace = self.label_height
        if platform.system() == "Windows":
            vspace += self.label_metrics.descent()
        y1 += vspace
        self.ylast = y1
        ## LCL-EL
        texts = [lcl_el, mean_lcl_el, srw_lcl_el]
        count = 5
        for text in texts:
            rect = QtCore.QRect(x1*count, y1, x1, self.label_height)
            qp.drawText(rect, QtCore.Qt.TextDontClip | QtCore.Qt.AlignRight, text)
            count += 2
        vspace = self.label_height
        if platform.system() == "Windows":
            vspace += self.label_metrics.descent()
        y1 += vspace
        self.ylast = y1
        ## Effective Shear
        texts = [efbwd, mean_ebw, srw_ebw]
        count = 5
        for text in texts:
            rect = QtCore.QRect(x1*count, y1, x1, self.label_height)
            qp.drawText(rect, QtCore.Qt.TextDontClip | QtCore.Qt.AlignRight, text)
            count += 2
        vspace = self.label_height + self.tpad
        if platform.system() == "Windows":
            vspace += self.label_metrics.descent()
        y1 += vspace
        self.ylast = y1
        ## BRN Shear and 4-6km SR Wind
        texts = [brn_shear, srw_4_5km]
        for text in texts:
            rect = QtCore.QRect(x1*5, y1, x1, self.label_height)
            qp.drawText(rect, QtCore.Qt.TextDontClip | QtCore.Qt.AlignRight, text)
            vspace = self.label_height 
            if platform.system() == "Windows":
                vspace += self.label_metrics.descent()
            y1 += vspace

        self.ylast = y1
        y1 += self.label_height + self.tpad # Not entirely sure why this doesn't 
                                            # need the extra pixels on Windows.
        ## bunkers motion
        texts = [bunkers_right, bunkers_left]
        colors =[QtGui.QColor('#0099CC'), QtGui.QColor('#FF6666')]
        for text, color in zip(texts, colors):
            rect = QtCore.QRect(x1*5, y1, x1, self.label_height)
            pen = QtGui.QPen(color, 1, QtCore.Qt.SolidLine)
            qp.setPen(pen)
            qp.drawText(rect, QtCore.Qt.TextDontClip | QtCore.Qt.AlignRight, text)
            vspace = self.label_height
            if platform.system() == "Windows":
                vspace += self.label_metrics.descent()
            y1 += vspace
        self.ylast = y1
        pen = QtGui.QPen(self.fg_color, 1, QtCore.Qt.SolidLine)
        qp.setPen(pen)
        ## upshear and downshear vectors
        texts = [downshear, upshear]
        for text in texts:
            rect = QtCore.QRect(x1*5, y1, x1, self.label_height)
            qp.drawText(rect, QtCore.Qt.TextDontClip | QtCore.Qt.AlignRight, text)
            if platform.system() == "Windows":
                vspace += self.label_metrics.descent()
            y1 += vspace

if __name__ == '__main__':
    app_frame = QtGui.QApplication([])    
    tester = plotKinematics()
    tester.show()    
    app_frame.exec_()



import numpy as np
import os
from qtpy import QtGui, QtCore, QtWidgets
import sharppy.sharptab as tab
from sharppy.sharptab.constants import *
import sharppy.databases.sars as sars
import platform

from datetime import datetime

## routine written by Kelton Halbert
## keltonhalbert@ou.edu

__all__ = ['backgroundAnalogues', 'plotAnalogues']

class backgroundAnalogues(QtWidgets.QFrame):
    '''
    Handles drawing the background frame for the
    SARS window.
    '''
    def __init__(self):
        ''' Calls the initUI function to initialize
            the background frame. Inherits from the
            QtWidgets.QFrame Object.
        '''
        super(backgroundAnalogues, self).__init__()
        self.initUI()

    def initUI(self):
        '''
        Initializes the frame.
        
        The frame is drawn on a QPixmap, 
        and is not rendered visible in this function.
        '''
        ## set the frame stylesheet
        self.setStyleSheet("QFrame {"
            "  background-color: rgb(0, 0, 0);"
            "  border-width: 1px;"
            "  border-style: solid;"
            "  border-color: #3399CC;}")
        ## Set the padding constants
        self.lpad = 3; self.rpad = 5
        self.tpad = 5; self.bpad = 5



        ## set the window metrics (height, width)
        self.wid = self.size().width()
        self.hgt = self.size().height()
        self.tlx = self.rpad; self.tly = self.tpad
        self.brx = self.wid; self.bry = self.hgt

        fsize1 = np.floor(.09 * self.hgt)
        fsize2 = np.floor(.07 * self.hgt)
        fsize3 = np.floor(.06 * self.hgt)

        self.tpad = np.floor(.03 * self.hgt)

        ## set various fonts
        self.title_font = QtGui.QFont('Helvetica', fsize1)
        self.plot_font = QtGui.QFont('Helvetica', fsize2)
        self.match_font = QtGui.QFont('Helvetica', fsize3)
        ## get the metrics on the fonts. This is used to get their size.
        self.title_metrics = QtGui.QFontMetrics( self.title_font )
        self.plot_metrics = QtGui.QFontMetrics( self.plot_font )
        self.match_metrics = QtGui.QFontMetrics( self.match_font )
        ## get the height of each font. This is so that we can do propper font aligning
        ## when drawing text

        if platform.system() == "Windows":
            fsize1 -= self.title_metrics.descent()
            fsize2 -= self.plot_metrics.descent()
            fsize3 -= self.match_metrics.descent()

            self.title_font = QtGui.QFont('Helvetica', fsize1)
            self.plot_font = QtGui.QFont('Helvetica', fsize2)
            self.match_font = QtGui.QFont('Helvetica', fsize3)
            ## get the metrics on the fonts. This is used to get their size.
            self.title_metrics = QtGui.QFontMetrics( self.title_font )
            self.plot_metrics = QtGui.QFontMetrics( self.plot_font )
            self.match_metrics = QtGui.QFontMetrics( self.match_font )
		
        self.title_height = self.title_metrics.xHeight() + self.tpad
        self.plot_height = self.plot_metrics.xHeight() + self.tpad
        self.match_height = self.match_metrics.xHeight() + self.tpad
        ## this variable gets set and used by other functions for knowing
        ## where in pixel space the last line of text ends
        self.ylast = self.tpad
        self.text_start = 0
        ## placement of supercell vs hail
        self.divide = (self.brx / 6)*3+10
        self.selectRect = None

        ## The widget will be drawn on a QPixmap
        self.plotBitMap = QtGui.QPixmap(self.width()-2, self.height()-2)
        self.plotBitMap.fill(self.bg_color)
        ## plot the background
        self.plotBackground()
    
    def draw_frame(self, qp):
        '''
        Draws the background frame and the text headers.
        '''
        ## initialize a white pen with thickness 1 and a solid line
        pen = QtGui.QPen(self.fg_color, 1, QtCore.Qt.SolidLine)
        qp.setPen(pen)
        
        ## make the initial x value relative to the width of the frame
        x1 = self.brx / 6
        self.ylast = self.tpad

        ## use the larger title font to plot the title, and then
        ## add to self.ylast the height of the font + padding
        qp.setFont(self.title_font)
        rect0 = QtCore.QRect(0, self.ylast, self.brx, self.title_height)
        qp.drawText(rect0, QtCore.Qt.TextDontClip | QtCore.Qt.AlignCenter,
            'SARS - Sounding Analogue System')
        self.ylast += (self.title_height + self.tpad)
        
        ## draw some lines to seperate the hail and supercell windows,
        ## then add to the running sum for vertical placement
        qp.drawLine(0, self.ylast, self.brx, self.ylast)
        qp.drawLine(self.brx/2, self.ylast, self.brx/2, self.bry)
        self.ylast += self.tpad
        
        ## plot the text for the hail and supercell windows using the running
        ## ylast sum
        qp.setFont(self.plot_font)
        rect1 = QtCore.QRect(x1*1, self.ylast, x1, self.plot_height)
        qp.drawText(rect1, QtCore.Qt.TextDontClip | QtCore.Qt.AlignCenter,
            'SUPERCELL')
            
        rect2 = QtCore.QRect(x1*4, self.ylast, x1, self.plot_height)
        qp.drawText(rect2, QtCore.Qt.TextDontClip | QtCore.Qt.AlignCenter,
            'SGFNT HAIL')
        ## Add to the running sum once more for future text
        self.ylast += (self.title_height)
        ## the hail and supercell windows both need to have a vertical starting reference
        self.text_start = self.ylast
    

    
    def resizeEvent(self, e):
        '''
        Handles when the window gets resized.
        '''
        self.initUI()

    def plotBackground(self):
        '''
        Handles drawing the text background.
        
        This draws onto a QPixmap.
        '''
        ## initialize a QPainter objext
        qp = QtGui.QPainter()
        qp.begin(self.plotBitMap)
        ## draw the frame
        self.draw_frame(qp)
        qp.end()


class plotAnalogues(backgroundAnalogues):
    updatematch = QtCore.Signal(str)
    '''
    Handles the non-background plotting
    of the SARS window. This inherits a
    backgroundAnalogues Object that
    takes care of drawing the frame
    background onto a QPixmap. This will
    inherit that QPixmap and continue 
    drawing on it, finally rendering the
    QPixmap via the function paintEvent.
    '''
    def __init__(self):
        '''
        Initializes the data needed for drawing the
        data onto the QPixmap. 
        
        Parameters
        ----------
        prof: a Profile object
        self.view.setDataSource(self.data_sources[self.model], self.run)
        
        '''
        ## get the surfce based, most unstable, and mixed layer
        ## parcels to use for indices, as well as the sounding
        ## profile itself.
        self.bg_color = QtGui.QColor('#000000')
        self.fg_color = QtGui.QColor('#ffffff')
        self.use_left = False
        super(plotAnalogues, self).__init__()
        self.prof = None 

    def setProf(self, prof):
        self.prof = prof

        if self.use_left:
            self.hail_matches = prof.left_matches
            self.sup_matches = prof.left_supercell_matches
        else:
            self.hail_matches = prof.right_matches
            self.sup_matches = prof.right_supercell_matches

        self.ybounds_hail = np.empty((len(self.hail_matches[0]),2))
        self.ybounds_sup = np.empty((len(self.sup_matches[0]),2))

        self.ylast = self.tpad
        self.clearData()
        self.plotBackground()
        self.plotData()
        self.update()


    def setPreferences(self, update_gui=True, **prefs):
        self.bg_color = QtGui.QColor(prefs['bg_color'])
        self.fg_color = QtGui.QColor(prefs['fg_color'])

        if update_gui:
            if self.use_left:
                self.hail_matches = self.prof.left_matches
                self.sup_matches = self.prof.left_supercell_matches
            else:
                self.hail_matches = self.prof.right_matches
                self.sup_matches = self.prof.right_supercell_matches

            self.ybounds_hail = np.empty((len(self.hail_matches[0]),2))
            self.ybounds_sup = np.empty((len(self.sup_matches[0]),2))

            self.clearData()
            self.plotBackground()
            self.plotData()
            self.update()


    def setDeviant(self, deviant):
        self.use_left = deviant == 'left'

        if self.use_left:
            self.hail_matches = self.prof.left_matches
            self.sup_matches = self.prof.left_supercell_matches
        else:
            self.hail_matches = self.prof.right_matches
            self.sup_matches = self.prof.right_supercell_matches

        self.ybounds_hail = np.empty((len(self.hail_matches[0]),2))
        self.ybounds_sup = np.empty((len(self.sup_matches[0]),2))

        self.clearData()
        self.plotBackground()
        self.plotData()
        self.update()


    def resizeEvent(self, e):
        '''
        Handles when the window is resized.
        
        Parameters
        ----------
        e: an Event object
        
        '''
        super(plotAnalogues, self).resizeEvent(e)
        ## if the window is resized, replot the data
        ## in the QPixmap.
        self.plotData()
    
    def paintEvent(self, e):
        '''
        Handles drawing the QPixmap onto the
        widget.
        
        Parameters
        ----------
        e: an Event object
        
        '''
        super(plotAnalogues, self).paintEvent(e)
        qp = QtGui.QPainter()
        qp.begin(self)
        qp.drawPixmap(1, 1, self.plotBitMap)
        qp.end()

    def clearData(self):
        '''
        Handles the clearing of the pixmap
        in the frame.
        '''
        self.plotBitMap = QtGui.QPixmap(self.width(), self.height())
        self.plotBitMap.fill(self.bg_color)

    def plotData(self):
        '''
        Handles the drawing of the SARS
        matches onto the QPixmap.
        '''
        if self.prof is None:
            return

        ## initialize a QPainter object
        qp = QtGui.QPainter()
        qp.begin(self.plotBitMap)
        ## draw the matches
        self.drawSARS(qp, type='HAIL')
        self.drawSARS(qp, type='TOR')
        qp.end()
    
    def drawSARS(self, qp, **kwargs):
        '''
        This draws the SARS matches.
        
        Parameters
        ----------
        qp: a QtGui.QPainter Object
        type: A string for the type of 
            SARS matches. 'HAIL' for 
            hail matches and 'TOR' for 
            supercell matches.
        '''
        x1 = self.brx / 6
        ## get the type of matches, and determine
        ## which side of the frame the matches will
        ## be plotted in.
        type = kwargs.get('type', 'HAIL')
        if type == 'TOR':
            self.matches = self.sup_matches
            sigstr = 'TOR'
            ## this is the horizontal placement
            ## of the text in the frame. Valid
            ## integers are from 0-6.
            place = 1
            ## the quality match date [0] and the type/size
            ## [1] palcement are set in this tuple.
            place2 = (self.lpad, (self.brx/2.) - x1 * 3./4. + 2)
            self.ybounds = self.ybounds_sup
        else:
            self.matches = self.hail_matches
            sigstr = 'SIG'
            ## this is the horizontal placement
            ## of the text in the frame. Valid
            ## integers are from 0-6.
            place = 4
            ## the quality match date [0] and the type/size
            ## [1] palcement are set in this tuple.
            place2 = (x1*3+7, x1*5.5-5)
            self.ybounds = self.ybounds_hail

        ## if there are no matches, leave the function to prevent crashing
        pen = QtGui.QPen(self.fg_color, 1, QtCore.Qt.SolidLine)
        if self.matches[0].size == 0:
            pen.setColor(self.fg_color)
            qp.setPen(pen)
            qp.setFont(self.match_font)
            ## draw the text 2/5 from the top
            rect2 = QtCore.QRect(x1*place, self.bry * (2./5.), x1, self.match_height)
            qp.drawText(rect2, QtCore.Qt.TextDontClip | QtCore.Qt.AlignCenter,
                'No Quality Matches')

        else:
            ## set the pen, font
            qp.setPen(pen)
            qp.setFont(self.plot_font)
            ## self.ylast has to be this way in order to plot relative to the bottom
            self.ylast = (self.bry - self.bpad*3)
            
            ## get various data to be plotted
            sig_prob = tab.utils.INT2STR( np.around( self.matches[-1]*100 ) )
            sig_str = 'SARS: ' + sig_prob + '% ' + sigstr
            num_matches = tab.utils.INT2STR( self.matches[-3] )
            match_str = '(' + num_matches + ' loose matches)'
            
            ## if there are more than 0 loose matches, draw
            ## draw the match statistics
            if self.matches[-3] > 0:
                qp.setFont(self.match_font)
                ## set the color of the font
                if self.matches[-1]*100. >= 50.:
                    pen.setColor(QtCore.Qt.magenta)
                    qp.setPen(pen)
                else:
                    pen.setColor(self.fg_color)
                    qp.setPen(pen)
                ## draw the text
                rect0 = QtCore.QRect(x1*place, self.ylast, x1, self.match_height)
                qp.drawText(rect0, QtCore.Qt.TextDontClip | QtCore.Qt.AlignCenter,
                            sig_str)
                ## since we start at the bottom and move up, subtract the height instead of add
                self.ylast -= (self.match_height + self.bpad)
                
                rect1 = QtCore.QRect(x1*place, self.ylast, x1, self.match_height)
                qp.drawText(rect1, QtCore.Qt.TextDontClip | QtCore.Qt.AlignCenter,
                    match_str)
            ## If not, don't do anything
            else:
                pass
            
            ## if there are no quality matches, let the gui know
            if len(self.matches[0]) == 0:
                pen.setColor(self.fg_color)
                qp.setPen(pen)
                qp.setFont(self.match_font)
                ## draw the text 2/5 from the top
                rect2 = QtCore.QRect(x1*place, self.bry * (2./5.), x1, self.match_height)
                qp.drawText(rect2, QtCore.Qt.TextDontClip | QtCore.Qt.AlignCenter,
                    'No Quality Matches')
            ## if there are more than 0 quality matches...
            else:
                pen.setColor(self.fg_color)
                qp.setPen(pen)
                qp.setFont(self.match_font)
                ## start the vertical sum at the reference point
                self.ylast = self.text_start
                idx  = 0
                ## loop through each of the matches
                i = 0
                for m in self.matches[0]:
                    mdate, mloc = m.split(".")
                    mdate = datetime.strptime(mdate, "%y%m%d%H").strftime("%d %b %y %HZ")
                    match_str = "%s (%s)" % (mdate, mloc)
                    ## these are the rectangles that matches will plot inside of
                    rect3 = QtCore.QRect(place2[0], self.ylast, x1, self.match_height)
                    rect4 = QtCore.QRect(place2[1], self.ylast, x1, self.match_height)
                    self.ybounds[i, 0] = self.ylast
                    self.ybounds[i, 1] = self.ylast + self.match_height
                    ## size or type is used for setting the color
                    size = self.matches[1][idx]
                    if type == 'TOR':
                        size_str = size[:-3]
                        if size.startswith('SIG'):
                            pen.setColor(QtGui.QColor(RED))
                            qp.setPen(pen)
                        elif size.startswith('WEAK'):
                            pen.setColor(QtGui.QColor(LBLUE))
                            qp.setPen(pen)
                        elif size.startswith('NON'):
                            pen.setColor(QtGui.QColor(LBROWN))
                            qp.setPen(pen)
                    else:
                        size_str = str( format(size, '.2f' ) )
                        if size >= 2.0:
                            pen.setColor(QtGui.QColor(RED))
                            qp.setPen(pen)
                        else:
                            pen.setColor(QtGui.QColor(LBLUE))
                            qp.setPen(pen)
                    ## draw the text
                    qp.drawText(rect3, QtCore.Qt.TextDontClip | QtCore.Qt.AlignLeft, match_str )
                    qp.drawText(rect4, QtCore.Qt.TextDontClip | QtCore.Qt.AlignLeft, size_str )
                    ## is there is a selected match, draw the bounds
                    if self.selectRect is not None:
                        pen.setColor(QtGui.QColor(LBLUE))
                        qp.setPen(pen)
                        topLeft = self.selectRect.topLeft()
                        topRight = self.selectRect.topRight()
                        bottomLeft = self.selectRect.bottomLeft()
                        bottomRight = self.selectRect.bottomRight()
                        qp.drawLine(topLeft, topRight)
                        qp.drawLine(bottomLeft, bottomRight)
                    idx += 1
                    i += 1
                    ## add to the running vertical sum
                    vspace = self.match_height
                    if platform.system() == "Windows":
                        vspace += self.match_metrics.descent()
                    self.ylast += vspace
            self.ylast = self.tpad

    def mousePressEvent(self, e):
        if self.prof is None or (len(self.sup_matches[0]) == 0 and len(self.hail_matches[0]) == 0):
            return

        pos = e.pos()
        ## is this a supercell match?
        if pos.x() < (self.brx / 2.):
            ## loop over supercells
            for i, bound in enumerate(self.ybounds_sup):
                if bound[0] < pos.y() and bound[1] > pos.y():
                    filematch = sars.getSounding(self.sup_matches[0][i], "supercell")
                    print(filematch)
                    self.updatematch.emit(filematch)
                    ## set the rectangle used for showing
                    ## a selected match
                    self.selectRect = QtCore.QRect(0, self.ybounds_sup[i, 0],
                                    self.brx / 2.,
                                    self.ybounds_sup[i, 1] - self.ybounds_sup[i, 0])
                    break
        ## is this a hail match?
        elif pos.x() > (self.brx / 2.):
            ## loop over hail
            for i, bound in enumerate(self.ybounds_hail):
                if bound[0] < pos.y() and bound[1] > pos.y():
                    filematch = sars.getSounding(self.hail_matches[0][i], "hail")
                    print(filematch)
                    self.updatematch.emit(filematch)
                    ## set the rectangle used for showing
                    ## a selected match
                    self.selectRect = QtCore.QRect(self.brx / 2., self.ybounds_hail[i, 0],
                                    self.brx / 2.,
                                    self.ybounds_hail[i, 1] - self.ybounds_hail[i, 0])
                    break
        self.clearData()
        self.plotBackground()
        self.plotData()
        self.update()
        #logging.debug("Calling plotAnaloges.parentWidget().setFocus()")
        self.parentWidget().setFocus()

    def setSelection(self, filematch):
        """
            Load in the SARS analog you've clicked.
        """
        match_name = os.path.basename(filematch)
#        print("\n\nSETSELECION:", match_name, filematch, self.sup_matches[0], self.hail_matches[0])
        sup_matches = [sars.getSounding(f, 'supercell').split('/')[-1] for f in self.sup_matches[0]] 
        hail_matches = [sars.getSounding(f, 'hail').split('/')[-1] for f in self.hail_matches[0]]
#        print(sup_matches, hail_matches) 
        if match_name in sup_matches:
            idx = np.where(np.asarray(sup_matches, dtype=str) == match_name)[0][0]
            lbx = 0.
            ybounds = self.ybounds_sup
        if match_name in hail_matches:
            idx = np.where(np.asarray(hail_matches, dtype=str) == match_name)[0][0]
            lbx = self.brx / 2.
            ybounds = self.ybounds_hail

        self.selectRect = QtCore.QRect(lbx, ybounds[idx, 0],
                        self.brx / 2., ybounds[idx, 1] - ybounds[idx, 0])
        self.clearData()
        self.plotBackground()
        self.plotData()
        self.update()
        self.parentWidget().setFocus()

    def clearSelection(self):
        self.selectRect = None

        self.clearData()
        self.plotBackground()
        self.plotData()
        self.update()
        #print(self.parent, self.parentWidget())
        #self.setParent(self.parent)
        self.parentWidget().setFocus()


if __name__ == '__main__':
    app_frame = QtGui.QApplication([])        
    #tester = plotText(['sfcpcl', 'mlpcl', 'mupcl'])
    tester = plotAnalogues()
    #tester.setProf()
    tester.show()        
    app_frame.exec_()



import numpy as np
from qtpy import QtGui, QtCore, QtWidgets
import sharppy.sharptab as tab
from sharppy.sharptab.constants import *
import platform

## routine written by Kelton Halbert
## keltonhalbert@ou.edu

__all__ = ['backgroundWinter', 'plotWinter']

class backgroundWinter(QtWidgets.QFrame):
    '''
    Handles drawing the background frame.
    '''
    def __init__(self):
        super(backgroundWinter, self).__init__()
        self.initUI()

    def initUI(self):
        ## initialize fram variables such as size,
        ## padding, etc.
        self.setStyleSheet("QFrame {"
            "  background-color: rgb(0, 0, 0);"
            "  border-width: 1px;"
            "  border-style: solid;"
            "  border-color: #3399CC;}")
        self.lpad = 5; self.rpad = 5
        self.tpad = 3; self.bpad = 3
        self.os_mod = 0
        self.barby = 0
        self.wid = self.size().width()
        self.hgt = self.size().height()
        self.tlx = self.rpad; self.tly = self.tpad
        self.brx = self.wid; self.bry = self.hgt

        if self.physicalDpiX() > 75:
            fsize = 8
        else:
            fsize = 10
        self.font_ratio = 0.0512
        self.label_font = QtGui.QFont('Helvetica', round(self.hgt * self.font_ratio))
        self.label_metrics = QtGui.QFontMetrics( self.label_font )

        if platform.system() == "Windows":
            self.os_mod = self.label_metrics.descent()

        self.label_height = self.label_metrics.xHeight() + self.tpad
        self.ylast = self.label_height

        self.plotBitMap = QtGui.QPixmap(self.width()-2, self.height()-2)
        self.plotBitMap.fill(self.bg_color)
        self.plotBackground()

    def draw_frame(self, qp):
        '''
        Draws the background frame and the text headers for indices.
        '''
        ## initialize a white pen with thickness 1 and a solid line
        pen = QtGui.QPen(self.dgz_color, 1, QtCore.Qt.SolidLine)
        qp.setPen(pen)
        qp.setFont(self.label_font)
        ## make the initial x value relative to the width of the frame
        x1 = self.brx / 10
        y1 = self.ylast + self.tpad
        ## draw the header
        rect1 = QtCore.QRect(0, self.tpad, self.wid, self.label_height)
        qp.drawText(rect1, QtCore.Qt.TextDontClip | QtCore.Qt.AlignCenter, '*** DENDRITIC GROWTH ZONE (-12 TO -17 C) ***')
        self.oprh_y1 = 2*self.label_height+self.tpad
        pen = QtGui.QPen(self.fg_color, 1, QtCore.Qt.SolidLine)
        qp.setPen(pen)
        sep = 5
        y1 = 3 * self.label_height + self.tpad + sep + self.os_mod
        self.layers_y1 = y1

        label = ['', '', '']
        for i in label:
            rect1 = QtCore.QRect(self.lpad, y1, self.wid/10, self.label_height)
            qp.drawText(rect1, QtCore.Qt.TextDontClip | QtCore.Qt.AlignLeft, i)
            y1 += self.label_height + sep + self.os_mod

        y1 = 3 * (self.label_height + self.os_mod) + self.tpad + sep + self.label_height + sep + self.os_mod
        begin = y1
        label = ['', '']
        for i in label:
            rect1 = QtCore.QRect(self.brx/2 + 2, y1, self.wid/10, self.label_height)
            qp.drawText(rect1, QtCore.Qt.TextDontClip | QtCore.Qt.AlignLeft, i)
            y1 += self.label_height + sep + self.os_mod

        qp.drawLine( 0, y1, self.brx, y1 )
        qp.drawLine( self.brx* .48, y1, self.brx*.48, begin )
        y1 = y1+3

        self.init_phase_y1 = y1

        label = ['']
        for i in label:
            rect1 = QtCore.QRect(self.lpad, y1, self.wid/10, self.label_height)
            qp.drawText(rect1, QtCore.Qt.TextDontClip | QtCore.Qt.AlignLeft, i)
            y1 += self.label_height + sep + self.os_mod
        qp.drawLine( 0, y1, self.brx, y1 )

        y1 = y1+3
        backup = y1
        label = ['','', '', '']
        for i in label:
            rect1 = QtCore.QRect(self.lpad, y1, self.wid/10, self.label_height)
            qp.drawText(rect1, QtCore.Qt.TextDontClip | QtCore.Qt.AlignLeft, i)
            y1 += self.label_height + sep + self.os_mod

        y1 = backup
        label = ['','', '', '']
        for i in label:
            rect1 = QtCore.QRect(self.brx/2 + 2, y1, self.wid/10, self.label_height)
            qp.drawText(rect1, QtCore.Qt.TextDontClip | QtCore.Qt.AlignLeft, i)
            y1 += self.label_height + sep + self.os_mod

        self.energy_y1 = backup
        qp.drawLine( 0, y1 + 0, self.brx, y1 + 0)
        qp.drawLine( self.brx* .48, y1 + 0, self.brx*.48, backup )
        y1 += 4
        rect1 = QtCore.QRect(0, y1, self.wid, self.label_height)
        qp.drawText(rect1, QtCore.Qt.TextDontClip | QtCore.Qt.AlignCenter, '*** BEST GUESS PRECIP TYPE ***')
        self.precip_type_y1 = y1 + self.label_height + 3 + 2 * self.os_mod
        self.ptype_tmpf_y1 = self.precip_type_y1 + self.label_height + 8 + 2 * self.os_mod

    def resizeEvent(self, e):
        '''
        Handles when the window gets resized.
        '''
        self.initUI()

    def plotBackground(self):
        '''
        Handles drawing the text background.
        '''
        ## initialize a QPainter objext
        qp = QtGui.QPainter()
        qp.begin(self.plotBitMap)
        ## draw the frame
        self.draw_frame(qp)
        qp.end()


class plotWinter(backgroundWinter):
    '''
    Handles plotting the indices in the frame.
    '''
    def __init__(self):
        ## get the surfce based, most unstable, and mixed layer
        ## parcels to use for indices, as well as the sounding
        ## profile itself.
        self.bg_color = QtGui.QColor('#000000')
        self.fg_color = QtGui.QColor('#ffffff')
        self.dgz_color = QtGui.QColor('#ffff00')

        super(plotWinter, self).__init__()
        self.prof = None

    def setProf(self, prof):
        self.prof = prof;

        # DGZ data
        self.dgz_pbot = prof.dgz_pbot
        self.dgz_ptop = prof.dgz_ptop
        self.dgz_zbot = tab.utils.M2FT(tab.interp.hght(prof, self.dgz_pbot))
        self.dgz_ztop = tab.utils.M2FT(tab.interp.hght(prof, self.dgz_ptop))
        self.dgz_depth = self.dgz_ztop - self.dgz_zbot
        self.oprh = prof.oprh
        self.dgz_meanrh = prof.dgz_meanrh
        self.dgz_pw = prof.dgz_pw
        self.dgz_meanq = prof.dgz_meanq
        self.dgz_meanomeg = prof.dgz_meanomeg

        # Inital Phase Types
        self.plevel = prof.plevel
        self.init_phase = prof.phase
        self.init_tmp = prof.tmp
        self.init_st = prof.st

        # TEMP Energy
        self.tpos = prof.tpos
        self.tneg = prof.tneg
        self.ttop = prof.ttop
        self.tbot = prof.tbot

        # WETBULB Energy
        self.wpos = prof.wpos
        self.wneg = prof.wneg
        self.wtop = prof.wtop
        self.wbot = prof.wbot

        # PRECIP TYPE
        self.precip_type = prof.precip_type

        # PRECIP TYPE SFC TEMPERATURE
        self.ptype_tmpf = tab.thermo.ctof(prof.tmpc[prof.get_sfc()])
        self.ptype_tmpf_string = "Based on SFC Temperature of %.2f F" % self.ptype_tmpf

        self.clearData()
        self.plotBackground()
        self.plotData()
        self.update()

    def setPreferences(self, update_gui=True, **prefs):
        self.bg_color = QtGui.QColor(prefs['bg_color'])
        self.fg_color = QtGui.QColor(prefs['fg_color'])
        self.dgz_color = QtGui.QColor(prefs['winter_dgz_color'])

        if update_gui:
            self.clearData()
            self.plotBackground()
            self.plotData()
            self.update()

    def resizeEvent(self, e):
        '''
        Handles when the window is resized.
        '''
        super(plotWinter, self).resizeEvent(e)
        self.plotData()

    def paintEvent(self, e):
        super(plotWinter, self).paintEvent(e)
        qp = QtGui.QPainter()
        qp.begin(self)
        qp.drawPixmap(1, 1, self.plotBitMap)
        qp.end()

    def clearData(self):
        '''
        Handles the clearing of the pixmap
        in the frame.
        '''
        self.plotBitMap = QtGui.QPixmap(self.width(), self.height())
        self.plotBitMap.fill(self.bg_color)

    def plotData(self):
        '''
        Handles the drawing of the text on the frame.
        '''
        if self.prof is None:
            return

        x1 = self.brx / 10
        y1 = self.bry / 19
        origin_x = x1*8.5
        origin_y = y1*15

        ## initialize a QPainter object
        qp = QtGui.QPainter()
        qp.begin(self.plotBitMap)
        qp.setRenderHint(qp.Antialiasing)
        qp.setRenderHint(qp.TextAntialiasing)
        ## draw the indices
        self.drawPrecipType(qp)
        self.drawPrecipTypeTemp(qp)
        self.drawOPRH(qp)
        self.drawInitial(qp)
        self.drawDGZLayer(qp)
        self.drawWCLayer(qp)
        qp.end()

    def drawOPRH(self, qp):
        if self.oprh < -.1 and tab.utils.QC(self.oprh) and self.dgz_meanomeg != -99990.0:
            pen = QtGui.QPen(QtCore.Qt.red, 1, QtCore.Qt.SolidLine)
        else:
            pen = QtGui.QPen(self.fg_color, 1, QtCore.Qt.SolidLine)
        qp.setPen(pen)
        qp.setFont(self.label_font)
        rect1 = QtCore.QRect(0, self.oprh_y1, self.wid, self.label_height)
        if self.dgz_meanomeg == -99990.0:
            qp.drawText(rect1, QtCore.Qt.TextDontClip | QtCore.Qt.AlignCenter, 'OPRH (Omega*PW*RH): N/A')
        else:
            qp.drawText(rect1, QtCore.Qt.TextDontClip | QtCore.Qt.AlignCenter, 'OPRH (Omega*PW*RH): ' + tab.utils.FLOAT2STR(self.oprh,2))

    def drawPrecipType(self, qp):
        big = QtGui.QFont('Helvetica', round(self.hgt * self.font_ratio) + 5)
        big_metrics = QtGui.QFontMetrics( big )
        height = big_metrics.xHeight() + self.tpad
        pen = QtGui.QPen(self.fg_color, 2, QtCore.Qt.SolidLine)
        qp.setPen(pen)
        qp.setFont(big)
        rect1 = QtCore.QRect(0, self.precip_type_y1, self.wid, height)
        qp.drawText(rect1, QtCore.Qt.TextDontClip | QtCore.Qt.AlignCenter, self.precip_type)

    def drawPrecipTypeTemp(self, qp):
        small = QtGui.QFont('Helvetica', round(self.hgt * self.font_ratio) -1)
        small_metrics = QtGui.QFontMetrics( small )
        height = small_metrics.xHeight() + self.tpad
        pen = QtGui.QPen(self.fg_color, 2, QtCore.Qt.SolidLine)
        qp.setPen(pen)
        qp.setFont(small)
        rect1 = QtCore.QRect(0, self.ptype_tmpf_y1, self.wid, height)
        qp.drawText(rect1, QtCore.Qt.TextDontClip | QtCore.Qt.AlignCenter, self.ptype_tmpf_string)

    def drawWCLayer(self, qp):
        sep = 5
        # Temperature Profile Stuff
        y1 = self.energy_y1
        x = self.lpad
        if self.tpos > 0 and self.tneg < 0:
            string = 'P/N: ' + str(round(self.tpos,0)) + ' / ' + str(round(self.tneg,0)) + ' J/kg'
            label = ['TEMPERATURE PROFILE', \
                     string, \
                     'Melt Lyr: ' + str(int(self.ttop)) + '-' + str(int(self.tbot)) + ' mb', \
                     'Frz Lyr: ' + str(int(self.tbot)) + '-' + str(int(self.prof.pres[self.prof.sfc])) + ' mb']
        else:
            label = ['TEMPERATURE PROFILE', '', 'Warm/Cold layers not found.', '']

        for i in label:
            rect1 = QtCore.QRect(x, y1, self.wid/10, self.label_height)
            qp.drawText(rect1, QtCore.Qt.TextDontClip | QtCore.Qt.AlignLeft, i)
            y1 += self.label_height + sep + self.os_mod

        # Wetbulb Profile stuff
        y1 = self.energy_y1
        x = self.brx/2 + 2
        if self.wpos > 0 and self.wneg < 0:
            string = 'P/N: ' + str(round(self.wpos,0)) + ' / ' + str(round(self.wneg,0)) + ' J/kg'
            label = ['WETBULB PROFILE', \
                     string, \
                     'Melt Lyr: ' + str(int(self.wtop)) + '-' + str(int(self.wbot)) + ' mb', \
                     'Frz Lyr: ' + str(int(self.wbot)) + '-' + str(int(self.prof.pres[self.prof.sfc])) + ' mb']
        else:
            label = ['WETBULB PROFILE', '', 'Warm/Cold layers not found.', '']

        for i in label:
            rect1 = QtCore.QRect(x, y1, self.wid/10, self.label_height)
            qp.drawText(rect1, QtCore.Qt.TextDontClip | QtCore.Qt.AlignLeft, i)
            y1 += self.label_height + sep + self.os_mod

    def drawDGZLayer(self, qp):
        pen = QtGui.QPen(self.fg_color, 1, QtCore.Qt.SolidLine)
        qp.setPen(pen)
        qp.setFont(self.label_font)
        sep = 5
        y1 = self.layers_y1
        label = ['Layer Depth: ' + tab.utils.INT2STR(self.dgz_depth) + " ft (" + tab.utils.INT2STR(self.dgz_zbot) + '-' +\
                 tab.utils.INT2STR(self.dgz_ztop) + ' ft msl)',\
                 'Mean Layer RH: ' + tab.utils.FLOAT2STR(self.dgz_meanrh,0) + ' %',\
                 'Mean Layer PW: ' + tab.utils.FLOAT2STR(self.dgz_pw,1) + ' in']
        for i in label:
            rect1 = QtCore.QRect(self.lpad, y1, self.wid/10, self.label_height)
            qp.drawText(rect1, QtCore.Qt.TextDontClip | QtCore.Qt.AlignLeft, i)
            y1 += self.label_height + sep + self.os_mod

        y1 = self.layers_y1 + self.label_height + sep + self.os_mod
        if self.dgz_meanomeg == 10*self.prof.missing:
            omeg = 'N/A'
        else:
            omeg = tab.utils.FLOAT2STR(self.dgz_meanomeg,1) + ' ub/s'
        label = ['Mean Layer MixRat: ' + tab.utils.FLOAT2STR(self.dgz_meanq,1) + ' g/kg', \
                 'Mean Layer Omega: ' + omeg]
        for i in label:
            rect1 = QtCore.QRect(self.brx/2 + 2, y1, self.wid/10, self.label_height)
            qp.drawText(rect1, QtCore.Qt.TextDontClip | QtCore.Qt.AlignLeft, i)
            y1 += self.label_height + sep + self.os_mod


    def drawInitial(self, qp):
        '''
        This handles the severe indices, such as STP, sig hail, etc.
        ---------
        qp: QtGui.QPainter object
        '''
        pen = QtGui.QPen(self.fg_color, 1, QtCore.Qt.SolidLine)
        qp.setPen(pen)
        qp.setFont(self.label_font)
        rect1 = QtCore.QRect(self.lpad, self.init_phase_y1,  self.wid/10, self.label_height)
        if self.plevel > 100:
            hght = tab.utils.M2FT(tab.interp.hght(self.prof, self.plevel))
            string = "Inital Phase: " + self.init_st + ' from: ' + tab.utils.INT2STR(self.plevel) + ' mb (' + tab.utils.INT2STR(hght) + ' ft msl; ' + tab.utils.FLOAT2STR(self.init_tmp,1) + ' C)'
        else:
            string = "Initial Phase:  No Precipitation layers found."
        qp.drawText(rect1, QtCore.Qt.TextDontClip | QtCore.Qt.AlignLeft, string)

if __name__ == '__main__':
    app_frame = QtGui.QApplication([])
    tester = plotWinter()
    tester.show()
    app_frame.exec_()
